#!/usr/bin/env python2.5
# -*- coding: UTF8 -*-
# Copyright (C) 2008 by Daniel Martin Yerga
# <dyerga@gmail.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Maemo WordPy: Blogging client for Internet Tablets
# Version 1.0
#

#import time
#start_time = time.time()

import gtk, gtk.glade
import hildon
import os
import gobject
import pango
import thread
import osso
import sys
from wordpy import settings, utils, io

#print 'modules: ', time.time() - start_time

#See if is running in scratchbox, and set the directories in consequence
if os.path.exists('/scratchbox/login'):
    scratchbox = True
else:
    scratchbox = False

accounts_db, settings_db, appdir, imgdir, glade_file, configdir, \
    logfile = settings.define_paths(scratchbox)

#print 'settings: ', time.time() - start_time

(ACCOUNTNAME, BLOGSYSTEM, DEFAULTACCOUNT) = range(3)
all_images = []

osso_c = osso.Context("net.yerga.maemowordpy", "1.0", False)

gtk.gdk.threads_init()

class WordpyUI:

    def __init__(self):   
        self.program = hildon.Program()
        self.program.__init__()
        gtk.set_application_name('')

        self.window = hildon.Window()
        self.window.set_title('Maemo WordPy')
        self.window.set_default_size(800, 480)
        self.window.connect("destroy", self.quit_and_save)
        self.window.connect("key-press-event", self.on_key_press)
        self.window.connect("window-state-event", self.on_window_state_change)
        self.window_in_fullscreen = False
        self.program.add_window(self.window)

        self.account_edited = False
        self.account_id = None
        self.common_option = None
        self.xml_file = None
        self.bloglist = []
        self.is_edited = False
        self.preserve_aspect = True
        self.resize_preserve_aspect = True
        self.ignore_preserve_aspect_callback = False
        self.cancel_media_upload = False
        self.edit_pixbuf = None
        self.screenshot_delay = 2

        #print 'before glade: ', time.time() - start_time
        
        self.glade=gtk.glade.XML(glade_file)
        gtkwindow = self.glade.get_widget("window1")
        vbox2 =  self.glade.get_widget("vbox2")
        vbox2.reparent(self.window)

        #print 'after glade: ', time.time() - start_time
                
        logo_img = self.glade.get_widget("logo_img")
        logo_img.set_from_file(imgdir + 'maemo-wordpy.png')
        self.recentposts = self.glade.get_widget("label1")
        self.recentposts_title = self.glade.get_widget("label204") 
        
        try:
            recentposts = RecentPosts()
            recentposts_txt = recentposts.get()
            self.recentposts.set_markup(recentposts_txt)
        except Exception, e:
            log = 'ERROR getting the recent posts info:\n%s' % e
            io.set_log_to_file(logfile, log)

        #print 'recentposts: ', time.time() - start_time
    
        ## Get the default settings and account
        ##[[name, blogsystem, user, password, blogid, blogname, url, enabled]]
        try:
            self.set_default_settings()
        except Exception, e:
            log = 'ERROR getting the default settings:\n%s' % e
            io.set_log_to_file(logfile, log)        

        #print 'default sett: ', time.time() - start_time

        if self.autosave_time != 0:
            self.autosave_timer = gobject.timeout_add (int(self.autosave_time*60000), \
                                                        self.check_autosave)

        self.edit_toolbar = self.glade.get_widget("hbox7")       

        # Main widgets #
        self.main_ntbk = self.glade.get_widget("notebook1")
        self.main_ntbk.connect("switch_page", self.on_notebook_changed)
        self.main_ntbk.set_show_tabs(False)

        self.home_btn = self.glade.get_widget("button17")
        self.home_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                0, 'home')

        create_post_btn = self.glade.get_widget("button43")
        create_post_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                1, None)

        manage_posts_btn = self.glade.get_widget("button44")
        manage_posts_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                4, 'manage_posts')

        manage_media_btn = self.glade.get_widget("button50") 
        manage_media_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                6, 'media')

        settings_btn = self.glade.get_widget("button52")
        settings_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                            7, 'settings')

        help_btn = self.glade.get_widget("button53")
        help_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                            8, None)

        about_btn = self.glade.get_widget("button54")
        about_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                            9, 'about')

        self.main_hbox = self.glade.get_widget("hbox6")
        self.main_hbox.set_no_show_all(True)
        self.main_hbox.hide()

        self.adv_opt_btn = self.glade.get_widget("button18")
        self.adv_opt_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                2, 'adv_opt')

        self.add_tags_btn = self.glade.get_widget("button19")
        self.add_tags_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                10, 'on_tags')
                
        self.publish_btn = self.glade.get_widget("button21")
        self.publish_btn.connect("clicked", self.publish_post)

        self.preview_btn = self.glade.get_widget("button20")
        self.preview_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                3, 'preview')

        self.main_btn = self.glade.get_widget("button47")
        self.main_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                1, None)

        ## Main post widgets ##
        self.txtPost = self.glade.get_widget("textview1")        
        self.postBuffer = self.txtPost.get_buffer()
                
        self.title_entry = self.glade.get_widget("entry1")     
        
        self.post_edited_btn = self.glade.get_widget("button168")
        self.post_edited_btn.connect("clicked", self.show_is_edited_post)
        post_edited_img = self.glade.get_widget("image6")
        post_edited_img.set_from_file(imgdir + 'post_edited.png') 
     
        post_sw = self.glade.get_widget("scrolledwindow1")
        hildon.hildon_helper_set_thumb_scrollbar(post_sw, True)

        ## Buttons for the edit toolbar ##

        bold_btn = self.glade.get_widget("button9")
        bold_btn.connect("clicked", self.insert_htmltags, 'bold', False) 

        italic_btn = self.glade.get_widget("button10")
        italic_btn.connect("clicked", self.insert_htmltags, 'italic', False) 
        
        add_img_btn = self.glade.get_widget("button11")
        add_img_btn.connect("clicked", self.show_add_images_page)
        
        link_btn = self.glade.get_widget("button12")
        link_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                        12, 'add_link')        

        underline_btn = self.glade.get_widget("button13")
        underline_btn.connect("clicked", self.insert_htmltags, 'underline', False)
        
        strikethrough_btn = self.glade.get_widget("button14")
        strikethrough_btn.connect("clicked", self.insert_htmltags, 'strikethrough', \
                                    False)
        
        paragraph_btn = self.glade.get_widget("button15")
        paragraph_btn.connect("clicked", self.insert_htmltags, 'paragraph', False)

        more_tags_btn = self.glade.get_widget("button16")
        more_tags_btn.connect("clicked", self.show_more_tags_page)
               
        left_btn = self.glade.get_widget("button113")
        left_btn.connect("clicked", self.insert_htmltags, 'left', True)
        right_btn = self.glade.get_widget("button114")
        right_btn.connect("clicked", self.insert_htmltags, 'right', True)
        center_btn = self.glade.get_widget("button115")
        center_btn.connect("clicked", self.insert_htmltags, 'center', True)
        justify_btn = self.glade.get_widget("button116")
        justify_btn.connect("clicked", self.insert_htmltags, 'justify', True)
        code_btn = self.glade.get_widget("button117")
        code_btn.connect("clicked", self.insert_htmltags, 'code', True)
        quote_btn = self.glade.get_widget("button118")
        quote_btn.connect("clicked", self.insert_htmltags, 'quote', True)
        li_btn = self.glade.get_widget("button119")
        li_btn.connect("clicked", self.insert_htmltags, 'li', True)
        ul_btn = self.glade.get_widget("button120")
        ul_btn.connect("clicked", self.insert_htmltags, 'ul', True)
        ol_btn = self.glade.get_widget("button121")
        ol_btn.connect("clicked", self.insert_htmltags, 'ol', True)  
        
        fontcolor_btn = self.glade.get_widget("button122")
        fontcolor_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                            12, 'fontcolor')
        
        bgcolor_btn = self.glade.get_widget("button123")
        bgcolor_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                            12, 'bgcolor')
        
        fontsize_btn = self.glade.get_widget("button124")
        fontsize_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                            12, 'fontsize')
          
        sup_btn = self.glade.get_widget("button125")
        sup_btn.connect("clicked", self.insert_htmltags, 'sup', True)
        sub_btn = self.glade.get_widget("button126")
        sub_btn.connect("clicked", self.insert_htmltags, 'sub', True)
        tt_btn = self.glade.get_widget("button127")
        tt_btn.connect("clicked", self.insert_htmltags, 'tt', True)
        cite_btn = self.glade.get_widget("button128")
        cite_btn.connect("clicked", self.insert_htmltags, 'cite', True)
        
        abbr_btn = self.glade.get_widget("button129")
        abbr_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                        12, 'add_abbr')
        acron_btn = self.glade.get_widget("button130")
        acron_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                        12, 'add_acronym')

        cancel_more_tags_btn = self.glade.get_widget("button112")
        cancel_more_tags_btn.connect("clicked", self.show_page_notebook, \
                                    self.main_ntbk, 1, None) 

        # Common dialog labels and cancel btn (used i.e. in add link)
        common_vbox = self.glade.get_widget("vbox42")
        
        self.fontcolor_chooser = hildon.ColorChooser()
        self.fontcolor_chooser.set_size_request(-1, 100)
        self.fontcolor_chooser.hide()
        common_vbox.pack_start(self.fontcolor_chooser, True, True, 0)
        common_vbox.reorder_child(self.fontcolor_chooser, 5)	
        
        self.hbox1_common = self.glade.get_widget("hbox73")
        self.hbox2_common = self.glade.get_widget("hbox74")
        self.fontsize_hbox = self.glade.get_widget("hbox82")
        
        self.common_entry = self.glade.get_widget("entry12")
        
        self.common_text_entry = self.glade.get_widget("entry13")
        
        self.label1_common = self.glade.get_widget("label130")
        self.label2_common = self.glade.get_widget("label129")
        
        self.fontsize_combo = self.glade.get_widget("comboboxentry6")
        
        self.set_common_btn = self.glade.get_widget("button110")
        self.set_common_btn.connect("clicked", self.on_common_btn_action)

        cancel_common_btn = self.glade.get_widget("button111")
        cancel_common_btn.connect("clicked", self.on_cancel_common)       

        ## Advanced Options widgets ##
        
        extended_sw = self.glade.get_widget("scrolledwindow3")
        hildon.hildon_helper_set_thumb_scrollbar(extended_sw, True)
               
        self.extended_frame = self.glade.get_widget("frame3")
        self.txtPostExt = self.glade.get_widget("textview3")        
        self.extendedBuffer = self.txtPostExt.get_buffer()
        
        other_opt_ntbk = self.glade.get_widget("notebook2")
        other_opt_ntbk.set_show_tabs(False)

        technorati_btn = self.glade.get_widget("button38")
        technorati_btn.connect("clicked", self.show_page_notebook, other_opt_ntbk, \
                                1, None)
        self.techtags_entry = self.glade.get_widget("entry2")
        techtags_apply_btn = self.glade.get_widget("button41")
        techtags_apply_btn.connect("clicked", self.show_page_notebook, \
                                    other_opt_ntbk, 0, None)
        techtags_cancel_btn = self.glade.get_widget("button42")
        techtags_cancel_btn.connect("clicked", self.show_page_notebook, \
                                    other_opt_ntbk, 0, None)
      
        self.trackbacks_btn = self.glade.get_widget("button39")
        self.trackbacks_btn.connect("clicked", self.show_page_notebook, \
                                    other_opt_ntbk, 2, None)
        self.tbacks_entry = self.glade.get_widget("entry4")
        tbacks_apply_btn = self.glade.get_widget("button45")
        tbacks_apply_btn.connect("clicked", self.show_page_notebook, \
                                    other_opt_ntbk, 0, None)
        tbacks_cancel_btn = self.glade.get_widget("button46") 
        tbacks_cancel_btn.connect("clicked", self.show_page_notebook, \
                                    other_opt_ntbk, 0, None)
        
        publish_opt_ntbk = self.glade.get_widget("notebook11")
        publish_opt_ntbk.set_show_tabs(False)

        publish_opt_btn = self.glade.get_widget("button40")
        publish_opt_btn.connect("clicked", self.show_page_notebook, \
                                publish_opt_ntbk, 0, 'show_pubopt')
                                
        self.publishopt_combo = self.glade.get_widget("comboboxentry7")
        self.publishopt_combo.child.set_property("editable", False) 
        self.publishopt_combo.set_active(0)
        self.comments_cb = self.glade.get_widget("checkbutton2") 
        
        self.pings_cb = self.glade.get_widget("checkbutton3")
        pubopt_apply_btn = self.glade.get_widget("button48")
        pubopt_apply_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                2, None)
        pubopt_cancel_btn = self.glade.get_widget("button49")
        pubopt_cancel_btn.connect("clicked", self.show_page_notebook, self.main_ntbk, \
                                2, None)              
        
        self.slug_entry = self.glade.get_widget("entry26")
        self.postpassword_entry = self.glade.get_widget("entry25")
        
        self.schedule_btn = self.glade.get_widget("button149")
        self.schedule_btn.connect("clicked", self.show_page_notebook, \
                                publish_opt_ntbk, 1, 'show_schedule')

        back_schedule_btn = self.glade.get_widget("button151")
        back_schedule_btn.connect("clicked", self.show_page_notebook, \
                                publish_opt_ntbk, 0, None)

        self.schedule_cbtn = self.glade.get_widget("checkbutton9")
        self.schedule_cbtn.set_active(False)

        self.date_schedule_entry = self.glade.get_widget("entry27")
        
        date_vbox = self.glade.get_widget("hbox102")
        calendar = hildon.Calendar()
        calendar.connect("day-selected", self.on_calendar_selected)
        calendar.set_size_request(410, -1)
        calendar.modify_font(pango.FontDescription("arial 14"))
        
        date_vbox.pack_start(calendar, False, False, 0)
        date_vbox.reorder_child(calendar, 0)
        calendar.show_all()

        hours_hbox = self.glade.get_widget("hbox11")
        minutes_hbox = self.glade.get_widget("hbox104")
        gobject.type_register(utils.FingerNE)
        
        self.hoursNE = utils.FingerNE('horizontal')
        self.hoursNE.set_maximum(23)     
        self.minutesNE = utils.FingerNE('horizontal')
        self.minutesNE.set_maximum(59)
 
        NE_list = [self.hoursNE, self.minutesNE]
        for i in range(len(NE_list)):
            NE_list[i].set_minimum(0)
            NE_list[i].set_size([60, 50, 60, 60])
            NE_list[i].set_image_buttons(imgdir + 'add.png', imgdir + \
                                        'remove.png')

        hours_hbox.pack_start(self.hoursNE, False, False, 0)
        hours_hbox.reorder_child(self.hoursNE, 1)
        minutes_hbox.pack_start(self.minutesNE, False, False, 0)
        minutes_hbox.reorder_child(self.minutesNE, 1)

        # Set actual date to the date widgets.
        try:
            self.schedule_widgets_to_actual_time()
        except Exception, e:
            log = 'ERROR setting the local time to the date widgets:\n%s' % e
            io.set_log_to_file(logfile, log) 

        ## Preview UI ##
        swin_preview = self.glade.get_widget("swin_preview")
        hildon.hildon_helper_set_thumb_scrollbar(swin_preview, True)
        
        try:
            import gtkhtml2
            self.htmlview = gtkhtml2.View()
            swin_preview.add(self.htmlview)
        except Exception, e:
            log = 'ERROR: gtkhtml2 module not loaded, preview not available:\n%s' % e
            io.set_log_to_file(logfile, log)         
            print "gtkhtml2 module not loaded."
            self.preview_btn.set_sensitive(False)

        ## Manage Posts UI ##
        self.manage_posts_ntbk = self.glade.get_widget("notebook3")
        self.manage_posts_ntbk.set_show_tabs(False)
        
        manage_posts_sw = self.glade.get_widget("scrolledwindow2")
        hildon.hildon_helper_set_thumb_scrollbar(manage_posts_sw, True) 
                      
        self.manage_posts_tv = self.glade.get_widget("treeview1")
        self.manage_posts_tv.set_headers_visible(True)
        self._manage_posts_tv_columns(self.manage_posts_tv)  
        self.manage_posts_model = self.__create_manage_posts_model('')
        self.manage_posts_tv.set_model(self.manage_posts_model)
        
        get_posts_to_manage_btn = self.glade.get_widget("button3")
        get_posts_to_manage_btn.connect("clicked", self.on_get_posts_to_manage) 
               
        clear_manage_posts_btn = self.glade.get_widget("button5")
        clear_manage_posts_btn.connect("clicked", self.on_clear_manage_posts) 

        edit_post_btn = self.glade.get_widget("button4")
        edit_post_btn.connect("clicked", self.on_edit_post)

        delete_post_btn = self.glade.get_widget("button160")
        delete_post_btn.connect("clicked", self.on_delete_post) 

        self.update_status_btn = self.glade.get_widget("button156")
        self.update_status_btn.connect("clicked", self.on_show_change_status_ui) 

        self.change_status_combo = self.glade.get_widget("comboboxentry12")
        self.change_status_combo.child.set_property("editable", False) 

        change_status_btn = self.glade.get_widget("button1")
        change_status_btn.connect("clicked", self.on_change_status_to_post)

        cancel_change_status_btn = self.glade.get_widget("button2")
        cancel_change_status_btn.connect("clicked", self.show_page_notebook, \
                                                self.manage_posts_ntbk, 0, None)
        
        ## Tags-Categories Interface ##
        
        self.tags_ntbk = self.glade.get_widget("notebook8")
        self.tags_ntbk.set_show_tabs(False)
        
        self.cats_btn = self.glade.get_widget("button65")
        self.cats_btn.connect("clicked", self.on_show_cats_ui)
                       
        tags_btn = self.glade.get_widget("button64")
        tags_btn.connect("clicked", self.on_show_tags_ui)

        self.tags_entry = self.glade.get_widget("entry9")
        
        recenttags_sw = self.glade.get_widget("scrolledwindow14")
        hildon.hildon_helper_set_thumb_scrollbar(recenttags_sw, True)
        
        self.recenttags_tv = self.glade.get_widget("treeview9")
        self.recenttags_model = self.__create_recenttags_model('')        
        self.recenttags_tv.set_model(self.recenttags_model)
        self._recenttags_tv_columns(self.recenttags_tv)       

        set_tags_btn = self.glade.get_widget("button66")
        set_tags_btn.connect("clicked", self.on_set_tags)

        unset_tags_btn = self.glade.get_widget("button70")
        unset_tags_btn.connect("clicked", self.on_unset_tags)
        
        cancel_cats_btn = self.glade.get_widget("button71")
        cancel_cats_btn.connect("clicked", self.on_cancel_cats)

        self.add_cat_hbox = self.glade.get_widget("hbox30")
        self.add_cat_entry = self.glade.get_widget("entry10")

        self.add_cat_btn = self.glade.get_widget("button68")
        self.add_cat_btn.connect("clicked", self.on_show_add_cat_ui)
        self.sync_cat_btn = self.glade.get_widget("button69")

        self.really_add_cat_btn = self.glade.get_widget("button72")
        self.really_add_cat_btn.connect("clicked", self.add_cat_to_server)     

        self.cancel_add_cat_btn = self.glade.get_widget("button73")
        self.cancel_add_cat_btn.connect("clicked", self.on_cancel_add_cat)

        cats_sw = self.glade.get_widget("scrolledwindow7")
        hildon.hildon_helper_set_thumb_scrollbar(cats_sw, True)

        self.categories_tv = self.glade.get_widget("treeview3")
        #self.categories_tv.set_headers_visible(True)
        
        self.cats_model = self.__create_cats_model('')        
        self.categories_tv.set_model(self.cats_model)
        self._cats_tv_columns(self.categories_tv)
        
        sync_cats_btn = self.glade.get_widget("button69")
        sync_cats_btn.connect("clicked", self.on_get_cats)

        set_cats_btn = self.glade.get_widget("button67")
        set_cats_btn.connect("clicked", self.on_set_cats_to_post)

        ## Add Images UI ##
        self.add_img_ntbk = self.glade.get_widget("notebook5")
        self.add_img_ntbk.set_show_tabs(False)
        self.add_img_ntbk.connect("switch_page", self.on_img_ntbk_changed)
        
        insert_img_btn = self.glade.get_widget("button29")
        insert_img_btn.connect("clicked", self.on_insert_img_to_post)
        
        self.local_btn = self.glade.get_widget("button23")
        self.local_btn.connect("clicked", self.show_page_notebook, self.add_img_ntbk, \
                            1, None)
        
        inet_btn = self.glade.get_widget("button24")
        inet_btn.connect("clicked", self.show_page_notebook, self.add_img_ntbk, \
                        2, None)
        
        flickr_btn = self.glade.get_widget("button25")
        flickr_btn.connect("clicked", self.show_page_notebook, self.add_img_ntbk, \
                        3, None)
        
        picasa_btn = self.glade.get_widget("button26")
        picasa_btn.connect("clicked", self.show_page_notebook, self.add_img_ntbk, \
                            4, None)

        self.back_imgntbk_btn = self.glade.get_widget("button74")
        self.back_imgntbk_btn.connect("clicked", self.show_page_notebook, \
                                        self.add_img_ntbk, 0, 'back_add_img')
        
        self.cache_img_btn = self.glade.get_widget("button27")
        self.cache_img_btn.connect("clicked", self.show_page_notebook, \
                                    self.add_img_ntbk, 7, 'cache_img')
        
        self.html_img_btn = self.glade.get_widget("button28")
        self.html_img_btn.connect("clicked", self.show_page_notebook, \
                                    self.add_img_ntbk, 5, 'html_img')
        
        cancel_add_img_btn = self.glade.get_widget("button30")
        cancel_add_img_btn.connect("clicked", self.on_cancel_insert_img)

        ## Local Images UI ##
        
        local_img_btn = self.glade.get_widget("button31")
        local_img_btn.connect("clicked", self.select_local_image)
        
        self.local_img_label = self.glade.get_widget("label45")
        self.local_img_label.set_text("")
        
        self.local_image_img = self.glade.get_widget("local_image_img")
        self.local_image_img.set_from_file(imgdir + 'img-missing.png')

        self.edit_img_btn = self.glade.get_widget("button32")
        self.edit_img_btn.connect("clicked", self.show_page_notebook, \
                                self.add_img_ntbk, 6, 'edit_img')

        ## Internet Images UI ##
        self.inet_img_entry = self.glade.get_widget("entry3")

        inet_img_btn = self.glade.get_widget("button57")
        inet_img_btn.connect("clicked", self.on_set_inet_img)

        ## Get Flickr photos UI ##
        self.flickr_photolist = gtk.ListStore(gobject.TYPE_STRING)        		
        self.combophotos = self.glade.get_widget("comboboxentry3")
        self.combophotos.set_model(self.flickr_photolist)
        self.combophotos.set_text_column(0)
        self.combophotos.connect("changed", self.on_photolist_changed)
        self.combophotos.child.set_property("editable", False)
        
        self.flickr_sizelist = gtk.ListStore(gobject.TYPE_STRING)
        self.combosize = self.glade.get_widget("comboboxentry2")
        self.combosize.set_model(self.flickr_sizelist)
        self.combosize.set_text_column(0)        
        self.combosize.connect("changed", self.on_sizelist_changed)
        self.combosize.child.set_property("editable", False)	
        
        flickrpage_align = self.glade.get_widget("alignment12")
        self.flickrpageNE = utils.FingerNE('horizontal')
        self.flickrpageNE.set_maximum(3000)
        self.flickrpageNE.set_value(1)
        self.flickrpageNE.set_minimum(1)
        self.flickrpageNE.set_size([60, 50, 60, 60])
        self.flickrpageNE.set_image_buttons(imgdir + 'add.png', imgdir + \
                                            'remove.png')
        flickrpage_align.add(self.flickrpageNE)

        self.labelWidth = self.glade.get_widget("label92")
        self.labelHeight = self.glade.get_widget("label93")        
        
        get_flickr_photos_btn = self.glade.get_widget("button83")
        get_flickr_photos_btn.connect("clicked", self.get_flickr_photolist)

        self.set_flickr_btn = self.glade.get_widget("button82")
        self.set_flickr_btn.connect("clicked", self.on_set_flickr_photo)	

        ## Get Picasa Photos UI ##
        
        self.picasa_albumlist = gtk.ListStore(gobject.TYPE_STRING)
        self.picasa_albums_cb = self.glade.get_widget("comboboxentry9")
        self.picasa_albums_cb.set_model(self.picasa_albumlist)   
        self.picasa_albums_cb.child.set_property("editable", False)
        
        for item in self.picasa_albums:
            self.picasa_albumlist.append([item])
        self.picasa_albums_cb.set_text_column(0)
        self.picasa_albums_cb.set_active(0)
        
        self.picasa_photolist = gtk.ListStore(gobject.TYPE_STRING)        
        self.picasa_photos_cb = self.glade.get_widget("comboboxentry8")
        self.picasa_photos_cb.connect("changed", self.on_picasaphotos_changed)
        self.picasa_photos_cb.set_model(self.picasa_photolist)
        self.picasa_photos_cb.set_text_column(0) 
        self.picasa_photos_cb.child.set_property("editable", False)                    
        
        get_picasa_photos_btn = self.glade.get_widget("button109")
        get_picasa_photos_btn.connect("clicked", self.get_picasa_photolist)
        
        self.picasa_width = self.glade.get_widget("label240")
        self.picasa_height = self.glade.get_widget("label241")
        
        self.set_picasa_btn = self.glade.get_widget("button169")
        self.set_picasa_btn.connect("clicked", self.on_set_picasa_photo)
        
        ## Edit Images UI ##
        self.edit_img = self.glade.get_widget("image13")
        self.edit_img.set_from_file(imgdir + 'img-missing.png') 

        self.edit_img_ntbk = self.glade.get_widget("notebook6")
        self.edit_img_ntbk.set_show_tabs(False)

        set_edited_img_btn = self.glade.get_widget("button154")
        set_edited_img_btn.connect("clicked", self.on_set_edited_img)
        
        crop_btn = self.glade.get_widget("button33")
        crop_btn.connect("clicked", self.show_page_notebook, self.add_img_ntbk, \
                            8, 'crop')
        resize_btn = self.glade.get_widget("button34")
        resize_btn.connect("clicked", self.show_page_notebook, self.edit_img_ntbk, \
                            1, 'resize_img')
        rotate_btn = self.glade.get_widget("button35")
        rotate_btn.connect("clicked", self.show_page_notebook, self.edit_img_ntbk, \
                            3, 'rotate_img')
        flip_btn = self.glade.get_widget("button36") 
        flip_btn.connect("clicked", self.show_page_notebook, self.edit_img_ntbk, \
                            4, None)
        screenshot_btn = self.glade.get_widget("button37") 
        screenshot_btn.connect("clicked", self.show_page_notebook, \
                            self.edit_img_ntbk, 5, None)
        filters_btn = self.glade.get_widget("button51")
        filters_btn.connect("clicked", self.show_page_notebook, \
                            self.edit_img_ntbk, 2, 'filters_img')

        do_resize_img_btn = self.glade.get_widget("button86")
        do_resize_img_btn.connect("clicked", self.on_resize_edit_img)        
        
        cancel_resize_btn = self.glade.get_widget("button87")
        cancel_resize_btn.connect("clicked", self.show_page_notebook, \
                                    self.edit_img_ntbk, 0, None)

        self.rotate_ntbk = self.glade.get_widget("notebook13")
        self.rotate_ntbk.set_show_tabs(False)
        
        rotate_angle_hbox = self.glade.get_widget("hbox57")

        rotate_left_btn = self.glade.get_widget("button91")
        rotate_left_btn.connect("clicked", self.on_rotate_img, 'left')   
        
        rotate_right_btn = self.glade.get_widget("button92")
        rotate_right_btn.connect("clicked", self.on_rotate_img, 'right')
        
        cancel_rotate_btn = self.glade.get_widget("button88")
        cancel_rotate_btn.connect("clicked", self.show_page_notebook, \
                                    self.edit_img_ntbk, 0, None)

        self.AngleNE = utils.FingerNE('horizontal')
        self.AngleNE.set_maximum(360)
        self.AngleNE.set_minimum(1)
        self.AngleNE.set_size([70, 50, 60, 60])
        self.AngleNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        rotate_angle_hbox.pack_start(self.AngleNE, False, False, 0)
        rotate_angle_hbox.reorder_child(self.AngleNE, 0)
        
        rotate_angle_btn = self.glade.get_widget("button90")
        rotate_angle_btn.connect("clicked", self.show_page_notebook, \
                                            self.rotate_ntbk, 1, 'rotate_angle')
        
        back_rotate_angle_btn = self.glade.get_widget("button93")
        back_rotate_angle_btn.connect("clicked", self.show_page_notebook, \
                                                      self.rotate_ntbk, 0, None)

        do_rotate_angle_btn = self.glade.get_widget("button89")
        do_rotate_angle_btn .connect("clicked", self.on_rotate_img, 'angle')       

        flip_vertically_btn = self.glade.get_widget("button96")
        flip_vertically_btn.connect("clicked", self.on_flip_img, 'vertical')
        
        flip_horizontally_btn = self.glade.get_widget("button95")
        flip_horizontally_btn.connect("clicked", self.on_flip_img, 'horizontal')

        cancel_flip_btn = self.glade.get_widget("button94")
        cancel_flip_btn.connect("clicked", self.show_page_notebook, \
                                self.edit_img_ntbk, 0, None)

        screenshot_hbox = self.glade.get_widget("hbox63")
        self.SnapTimeNE = utils.FingerNE('horizontal')
        self.SnapTimeNE.set_maximum(3600)
        self.SnapTimeNE.set_minimum(0)
        self.SnapTimeNE.set_size([60, 50, 60, 60])
        self.SnapTimeNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        screenshot_hbox.pack_start(self.SnapTimeNE, False, False, 0)
        screenshot_hbox.reorder_child(self.SnapTimeNE, 0)    

        get_shoot_btn = self.glade.get_widget("button97")
        get_shoot_btn.connect("clicked", self.get_screenshot)

        cancel_screenshot_btn = self.glade.get_widget("button98")
        cancel_screenshot_btn.connect("clicked", self.show_page_notebook, \
                                        self.edit_img_ntbk, 0, None)

        self.color_bright_hbox = self.glade.get_widget("hbox69")
       
        self.colorize_btn = self.glade.get_widget("button101")
        self.colorize_btn.connect("clicked", self.show_color_bright_hbox, True, \
                                'colorize')  
        
        self.brightness_btn = self.glade.get_widget("button102")              
        self.brightness_btn.connect("clicked", self.show_color_bright_hbox, True, \
                                    'brightness')

        self.colorizeBar = hildon.Controlbar()
        self.colorizeBar.set_min(0)
        self.colorizeBar.set_max(20)
        self.colorizeBar.set_value(5)
        self.colorizeBar.show()
        self.color_bright_hbox.pack_start(self.colorizeBar, False, False, 0)
        self.color_bright_hbox.reorder_child(self.colorizeBar, 0)		
        
        color_bright_back_btn = self.glade.get_widget("button105")
        color_bright_back_btn.connect("clicked", self.show_color_bright_hbox, \
                                    False, None)
        
        cancel_filters_btn = self.glade.get_widget("button103")
        cancel_filters_btn.connect("clicked", self.show_page_notebook, \
                                    self.edit_img_ntbk, 0, None)

        set_color_bright_btn = self.glade.get_widget("button104")
        set_color_bright_btn.connect("clicked", self.on_color_or_bright)

        ## Resize UI ##
        resize_width_hbox = self.glade.get_widget("hbox54")
        resize_height_hbox = self.glade.get_widget("hbox55")

        self.ResizeWidthNE = utils.FingerNE('horizontal')
        self.ResizeWidthNE.set_maximum(10000)
        self.ResizeWidthNE.set_minimum(1)
        self.ResizeWidthNE.set_size([90, 50, 60, 60])
        self.ResizeWidthNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        resize_width_hbox.pack_start(self.ResizeWidthNE, False, False, 0)
        resize_width_hbox.reorder_child(self.ResizeWidthNE, 1)        

        self.ResizeHeightNE = utils.FingerNE('horizontal')
        self.ResizeHeightNE.set_maximum(10000)
        self.ResizeHeightNE.set_minimum(1)
        self.ResizeHeightNE.set_size([90, 50, 60, 60])
        self.ResizeHeightNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        resize_height_hbox.pack_start(self.ResizeHeightNE, False, False, 0)
        resize_height_hbox.reorder_child(self.ResizeHeightNE, 1)
        
        self.ResizeWidthNE.entry.connect('changed', self.preserve_image_aspect, \
                                            "width", self.ResizeHeightNE.entry, \
                                            'resize')
        self.ResizeHeightNE.entry.connect('changed', self.preserve_image_aspect, \
                                            "height", self.ResizeWidthNE.entry, \
                                            'resize')

        resize_ratio_btn = self.glade.get_widget("checkbutton8")
        resize_ratio_btn.set_active(self.resize_preserve_aspect)
        resize_ratio_btn.connect('toggled', self.aspect_ratio_toggled, \
                                    self.ResizeWidthNE.entry, \
                                    self.ResizeHeightNE.entry, 'resize')
        

        ## Crop UI ##
        cancel_crop_btn = self.glade.get_widget("button85")
        cancel_crop_btn.connect("clicked", self.show_page_notebook, \
                                self.add_img_ntbk, 6, 'cancel_crop')
                                
        do_crop_btn = self.glade.get_widget("button84")
        do_crop_btn.connect("clicked", self.on_crop_img)


        self.crop_draw = self.glade.get_widget("drawingarea1")
        self.crop_draw.set_events(gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.POINTER_MOTION_HINT_MASK | gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_MOTION_MASK | gtk.gdk.BUTTON_RELEASE_MASK)

        ## HTML tags for images UI ##      
        width_hbox = self.glade.get_widget("hbox40")
        self.WidthNE = utils.FingerNE('horizontal')
        self.WidthNE.set_maximum(10000)
        self.WidthNE.set_minimum(0)
        self.WidthNE.set_size([90, 50, 60, 60])
        self.WidthNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        width_hbox.pack_start(self.WidthNE, False, False, 0)
        width_hbox.reorder_child(self.WidthNE, 1)        

        height_hbox = self.glade.get_widget("hbox41")
        self.HeightNE = utils.FingerNE('horizontal')
        self.HeightNE.set_maximum(10000)
        self.HeightNE.set_minimum(0)
        self.HeightNE.set_size([90, 50, 60, 60])
        self.HeightNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        height_hbox.pack_start(self.HeightNE, False, False, 0)
        height_hbox.reorder_child(self.HeightNE, 1)
        
        self.WidthNE.entry.connect('changed', self.preserve_image_aspect, "width", \
                                    self.HeightNE.entry, 'html')
        self.HeightNE.entry.connect('changed', self.preserve_image_aspect, "height", \
                                    self.WidthNE.entry, 'html')
		
        self.original_button = self.glade.get_widget("radiobutton6")
        self.original_button.set_active(True)
        self.original_button.connect('toggled', self.on_original_active)
        self.resize_button = self.glade.get_widget("radiobutton7")
        self.resize_button.set_group(self.original_button)
               
        self.img_align = self.glade.get_widget("comboboxentry1")
        self.img_align.child.set_property("editable", False)	

        self.en_alttext = self.glade.get_widget("entry11")
        
        self.labelwidth_html = self.glade.get_widget("label86")
        self.labelheight_html = self.glade.get_widget("label87")
		
        self.ratio_btn = self.glade.get_widget("checkbutton7")
        self.ratio_btn.set_active(self.preserve_aspect)
        self.ratio_btn.connect('toggled', self.aspect_ratio_toggled, self.WidthNE.entry, \
                                self.HeightNE.entry, 'html')        
        
        self.do_resize_widgets_insensitive()

        ## Cached images UI ##
        cache_sw = self.glade.get_widget("scrolledwindow10")
        hildon.hildon_helper_set_thumb_scrollbar(cache_sw, True)

        self.cache_model = self.cache_images_model('')

        self.cache_tv = self.glade.get_widget("treeview5")
        self.cache_tv.set_headers_visible(True)
        self.cache_tv.set_model(self.cache_model)
        self.__add_columns_cache_images(self.cache_tv)

        delete_cache_btn = self.glade.get_widget("button108")
        delete_cache_btn.connect("clicked", self.delete_cache_img)
        
        view_cache_img_btn = self.glade.get_widget("button58")
        view_cache_img_btn.connect("clicked", self.view_cache_img)
        
        ## Manage Media UI ##

        self.media_ntbk = self.glade.get_widget("notebook4")
        self.media_ntbk.connect("switch_page", self.on_media_ntbk_changed)
        self.media_ntbk.set_show_tabs(False)
        
        media_settings_btn = self.glade.get_widget("button164")
        media_settings_btn.connect("clicked", self.show_page_notebook, \
                                    self.main_ntbk, 7, 'media_settings')
        
        upload_flickr_btn = self.glade.get_widget("button55")
        upload_flickr_btn.connect("clicked", self.show_page_notebook, \
                                    self.media_ntbk, 1, 'flickr')
        
        upload_picasa_btn = self.glade.get_widget("button56")
        upload_picasa_btn.connect("clicked", self.show_page_notebook, \
                                    self.media_ntbk, 1, 'picasa')

        imagelist_sw = self.glade.get_widget("scrolledwindow4")
        hildon.hildon_helper_set_thumb_scrollbar(imagelist_sw, True)
        
        self.imagelist_tv = self.glade.get_widget("treeview2")
        self.imagelist_tv.set_rubber_banding(True)
        selection = self.imagelist_tv.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE) 
        self.imglist_model = self.__create_imglist_model('')
        self.imagelist_tv.set_model(self.imglist_model)
        self._imglist_tv_columns(self.imagelist_tv)
        
        add_imagefiles_img = self.glade.get_widget("image1")
        add_imagefiles_img.set_from_file(imgdir + 'add-big.png')
        
        add_imagefiles_btn = self.glade.get_widget("button7")
        add_imagefiles_btn.connect("clicked", self.add_img_to_list)

        del_imagefiles_img = self.glade.get_widget("image2")
        del_imagefiles_img.set_from_file(imgdir + 'remove.png')

        del_imagefiles_btn = self.glade.get_widget("button59")
        del_imagefiles_btn .connect("clicked", self.delete_img_from_list)

        desc_imagefiles_img = self.glade.get_widget("image3")
        desc_imagefiles_img.set_from_file(imgdir + 'edit.png')

        desc_img_btn = self.glade.get_widget("button8")
        desc_img_btn.connect("clicked", self.show_img_description)

        upload_imagefiles_img = self.glade.get_widget("image4")
        upload_imagefiles_img.set_from_file(imgdir + 'up.png')

        upload_img_btn = self.glade.get_widget("button22")
        upload_img_btn.connect("clicked", self.on_upload_images)
        
        cancel_media_img = self.glade.get_widget("image5")
        cancel_media_img.set_from_file(imgdir + 'cancel.png')
        cancel_media_btn = self.glade.get_widget("button163")
        cancel_media_btn.connect("clicked", self.show_page_notebook, \
                                self.media_ntbk, 0, None)

        cancel_desc_img_btn = self.glade.get_widget("button62")
        cancel_desc_img_btn.connect("clicked", self.show_page_notebook, \
                                self.media_ntbk, 1, 'cancel_desc_img')
        
        save_desc_img_btn = self.glade.get_widget("button60")
        save_desc_img_btn.connect("clicked", self.save_img_description)

        desc_flickr_ntbk = self.glade.get_widget("notebook7")
        desc_flickr_ntbk.set_show_tabs(False)
        desc_flickr_ntbk.connect("switch_page", self.on_desc_flickr_ntbk_changed) 
               
        self.img_title_entry = self.glade.get_widget("entry6")
        self.img_tags_entry = self.glade.get_widget("entry5")
        
        img_desc_textview = self.glade.get_widget("textview2")
        img_desc_textview.modify_font(pango.FontDescription("arial 16"))
        self.img_desc_buffer = img_desc_textview.get_buffer()
        self.batch_flickr_label = self.glade.get_widget("label139")
        
        
        self.public_img_rb = self.glade.get_widget("radiobutton4")
        self.public_img_rb.set_active(True)
        self.public_img_rb.connect('toggled', self.on_public_img_active)
        self.private_img_rb = self.glade.get_widget("radiobutton5")
        self.private_img_rb.set_group(self.public_img_rb)
        self.friend_img_rb = self.glade.get_widget("checkbutton4")
        self.family_img_rb = self.glade.get_widget("checkbutton5")
        
        self.do_private_img_widgets_insensitive()
        
        self.sets_btn = self.glade.get_widget("button75")
        self.sets_btn.connect("clicked", self.show_page_notebook, \
                                    desc_flickr_ntbk, 1, None)
        
        self.privacy_btn = self.glade.get_widget("button76")
        self.privacy_btn.connect("clicked", self.show_page_notebook, \
                                    desc_flickr_ntbk, 2, None)
        
        desc_img_sw = self.glade.get_widget("scrolledwindow5")
        hildon.hildon_helper_set_thumb_scrollbar(desc_img_sw, True)
        
        self.description_btn = self.glade.get_widget("button77")
        self.description_btn.connect("clicked", self.show_page_notebook, \
                                    desc_flickr_ntbk, 0, None)
        
        self.get_sets_btn = self.glade.get_widget("button78")
        self.get_sets_btn.connect("clicked", self.on_get_flickr_sets)

        sets_sw = self.glade.get_widget("scrolledwindow8")
        hildon.hildon_helper_set_thumb_scrollbar(sets_sw, True) 
               
        self.sets_tv = self.glade.get_widget("treeview4")
        self.sets_tv.set_headers_visible(True)
        self._sets_tv_columns(self.sets_tv)  
        self.sets_model = self.__create_sets_model('')  
        self.sets_tv.set_model(self.sets_model)

        self.upinfo_title = self.glade.get_widget("label119")
        self.upinfo_filename = self.glade.get_widget("label215")
        self.upinfo_pgbar = self.glade.get_widget("progressbar1")
        self.cancel_upinfo_btn = self.glade.get_widget("button79")
        #self.cancel_upinfo_btn.connect("clicked", self.on_cancel_upload_media)
        self.back_upinfo_btn = self.glade.get_widget("button80")
        self.back_upinfo_btn.connect("clicked", self.show_page_notebook, \
                                    self.media_ntbk, 0, None)        
        
        ## Upload to Picasa UI ##
        picasa_desc_sw = self.glade.get_widget("scrolledwindow15")
        hildon.hildon_helper_set_thumb_scrollbar(picasa_desc_sw, True)
        
        self.picasa_title = self.glade.get_widget("entry31")
        self.picasa_tags = self.glade.get_widget("entry30")

        self.picasa_album_cb = self.glade.get_widget("comboboxentry10")
        self.picasa_album_cb.set_model(self.picasa_albumlist)        
        self.picasa_album_cb.child.set_editable(False)
        self.picasa_album_cb.set_text_column(0)
        self.picasa_album_cb.set_active(0)
        
        self.batch_picasa_label = self.glade.get_widget("label143")
        
        picasa_desc_txtv = self.glade.get_widget("textview6")
        self.picasa_desc_buffer = picasa_desc_txtv.get_buffer()
        
        picasa_save_desc_btn = self.glade.get_widget("button170")
        picasa_save_desc_btn.connect("clicked", self.save_img_description)
        cancel_picasa_desc_btn = self.glade.get_widget("button171")
        cancel_picasa_desc_btn.connect("clicked", self.show_page_notebook, \
                                            self.media_ntbk, 1, 'cancel_desc_img')
        
        ## Settings UI ##
        self.settings_ntbk = self.glade.get_widget("notebook9")
        self.settings_ntbk.set_show_tabs(False)
        
        accounts_btn = self.glade.get_widget("button131")
        accounts_btn.connect("clicked", self.show_page_notebook, \
                                self.settings_ntbk, 1, None)

        general_settings_btn = self.glade.get_widget("button132")
        general_settings_btn.connect("clicked", self.show_page_notebook, \
                                    self.settings_ntbk, 2, None)
        
        plugins_btn = self.glade.get_widget("button148")
        plugins_btn.connect("clicked", self.show_page_notebook, \
                                self.settings_ntbk, 4, None)

        media_settings_btn = self.glade.get_widget("button167")
        media_settings_btn.connect("clicked", self.show_page_notebook, \
                                    self.settings_ntbk, 5, None)      

        ## General Settings UI ##
        fontsize_config_hbox = self.glade.get_widget("hbox110")

        self.fontsizeNE = utils.FingerNE('horizontal')
        self.fontsizeNE.entry.connect("changed", self.on_setting_change, 'fontsize')
        self.fontsizeNE.set_maximum(30)
        self.fontsizeNE.set_value(int(self.fontsize))
        self.fontsizeNE.set_minimum(9)
        self.fontsizeNE.set_size([60, 50, 60, 60])
        self.fontsizeNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')

        fontsize_config_hbox.pack_start(self.fontsizeNE, False, False, 0)
        fontsize_config_hbox.reorder_child(self.fontsizeNE, 1)

        self.as_time_hbox = self.glade.get_widget("hbox124")
        self.as_path_hbox = self.glade.get_widget("hbox137")
         
        self.as_path_entry = self.glade.get_widget("entry32")
        self.as_path_entry.set_text(self.autosave_path)
        as_path_btn = self.glade.get_widget("button150")
        as_path_btn.connect("clicked", self.on_select_autosave_path)
        
        self.astimeNE = utils.FingerNE('horizontal')
        self.astimeNE.entry.connect("changed", self.on_setting_change, 'astime')        
        self.astimeNE.set_maximum(60)
        self.astimeNE.set_value(int(self.autosave_time))
        self.astimeNE.set_minimum(1)
        self.astimeNE.set_size([60, 50, 60, 60])
        self.astimeNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')

        self.as_time_hbox.pack_start(self.astimeNE, False, False, 0)
        self.as_time_hbox.reorder_child(self.astimeNE, 1)

        self.autosave_cb = self.glade.get_widget("checkbutton1")
        self.autosave_cb.connect('toggled', self.on_autosave_cb_toggled)
        
        #print self.autosave_on
        if self.autosave_on == 1:
            self.autosave_cb.set_active(True)
        else:
            self.autosave_cb.set_active(False)
       
        if not self.autosave_cb.get_active():
            self.as_time_hbox.set_sensitive(False)
            self.as_path_hbox .set_sensitive(False) 
                 
        numposts_hbox = self.glade.get_widget("hbox125")
        self.numpostsNE = utils.FingerNE('horizontal')
        self.numpostsNE.entry.connect("changed", self.on_setting_change, 'numposts')         
        self.numpostsNE.set_maximum(5000)
        self.numpostsNE.set_minimum(1)
        self.numpostsNE.set_value(int(self.numposts))        
        self.numpostsNE.set_size([60, 50, 60, 60])
        self.numpostsNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')

        numposts_hbox.pack_start(self.numpostsNE, False, False, 0)
        numposts_hbox.reorder_child(self.numpostsNE, 1)
    
        ## Media settings UI ##
        auth_flickr_btn = self.glade.get_widget("button162")
        auth_flickr_btn.connect("clicked", self.on_authotize_to_flickr)

        flickr_settings_hbox = self.glade.get_widget("hbox114")
        self.perpageNE = utils.FingerNE('horizontal')
        self.perpageNE.entry.connect("changed", self.on_setting_change, 'perpage')         
        self.perpageNE.set_maximum(30)
        self.perpageNE.set_value(self.perpage)
        self.perpageNE.set_minimum(1)
        self.perpageNE.set_size([60, 50, 60, 60])
        self.perpageNE.set_image_buttons(imgdir + 'add.png', imgdir + 'remove.png')
        flickr_settings_hbox.pack_start(self.perpageNE, False, False, 0)
        flickr_settings_hbox.reorder_child(self.perpageNE, 1)

        self.picasa_login_entry = self.glade.get_widget("entry28")
        self.picasa_login_entry.connect("changed", self.on_setting_change, 'picasa_login')         
        self.picasa_login_entry.set_text(self.picasa_login)
        self.picasa_login_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')
        self.picasa_pass_entry = self.glade.get_widget("entry29")
        self.picasa_pass_entry.connect("changed", self.on_setting_change, 'picasa_pass')         
        self.picasa_pass_entry.set_text(self.picasa_pass)
        self.picasa_pass_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')

        get_picasa_albums_settings_btn = self.glade.get_widget("button172")
        get_picasa_albums_settings_btn.connect("clicked", self.get_picasa_albumlist)

        self.picasa_default_album_cb = self.glade.get_widget("comboboxentry11")        
        self.picasa_default_album_cb.set_model(self.picasa_albumlist)        
        self.picasa_default_album_cb.child.set_editable(False)
        self.picasa_default_album_cb.set_text_column(0)
        self.picasa_default_album_cb.set_active(0)

        ## Accounts UI ##        
        accounts_sw = self.glade.get_widget("scrolledwindow11")
        hildon.hildon_helper_set_thumb_scrollbar(accounts_sw, True)
                
        self.accounts_tv = self.glade.get_widget("treeview6")
        self.accounts_tv.set_headers_visible(True)
        
        self.accounts_model = self.create_accounts_list()
        self.accounts_tv.set_model(self.accounts_model)

        self.add_accounts_tv_columns(self.accounts_tv)

        new_account_btn = self.glade.get_widget("button133")
        new_account_btn.connect("clicked", self.show_page_notebook, \
                                self.settings_ntbk, 3, 'new_account')

        edit_account_btn = self.glade.get_widget("button134")
        edit_account_btn.connect("clicked", self.edit_account)
        
        delete_account_btn = self.glade.get_widget("button135")
        delete_account_btn.connect("clicked", self.delete_account)
        
        make_default_btn = self.glade.get_widget("button136")
        make_default_btn.connect("clicked", self.make_default_account)
        
        ## New account UI ##

        self.new_acc_wizard_ntbk = self.glade.get_widget("notebook10")
        self.new_acc_wizard_ntbk.set_show_tabs(False)
        self.new_acc_wizard_ntbk.connect("switch_page", \
                                            self.on_new_acc_wizard_ntbk_changed)

        self.blogging_system_cbox = self.glade.get_widget("comboboxentry4")
        self.blogging_system_cbox.child.set_property("editable", False)

        self.account_name_entry = self.glade.get_widget("entry14")
        self.blogurl_entry = self.glade.get_widget("entry15")
        self.blogurl_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')
        self.wordpress_user_entry = self.glade.get_widget("entry16")
        self.wordpress_user_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')
        self.wordpress_pass_entry  = self.glade.get_widget("entry17") 
        self.wordpress_pass_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')
        self.wordpress_url_hbox = self.glade.get_widget("hbox85")  

        self.blogger_user_entry  = self.glade.get_widget("entry18")
        self.blogger_user_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')
        self.blogger_pass_entry  = self.glade.get_widget("entry19")
        self.blogger_pass_entry.set_property('hildon-input-mode', \
                                                'HILDON_GTK_INPUT_MODE_FULL')

        self.blogger_list = gtk.ListStore(gobject.TYPE_STRING)
        self.blogger_blog_cbox = self.glade.get_widget("comboboxentry5")
        self.blogger_blog_cbox.set_model(self.blogger_list)
        self.blogger_blog_cbox.child.set_property("editable", False)

        get_blogger_blogs_btn = self.glade.get_widget("button141")
        get_blogger_blogs_btn.connect("clicked", self.get_blogger_bloglist)

        self.back_new_account_btn = self.glade.get_widget("button137")
        self.back_new_account_btn.connect("clicked", self.new_account_wizard, 'back')
        
        self.next_new_account_btn = self.glade.get_widget("button138")
        self.next_new_account_btn.connect("clicked", self.new_account_wizard, 'next')

        cancel_new_account_btn = self.glade.get_widget("button140")
        cancel_new_account_btn.connect("clicked", self.show_page_notebook, \
                                        self.settings_ntbk, 1, None)

        finish_new_account_btn = self.glade.get_widget("button139")
        finish_new_account_btn.connect("clicked", self.finish_new_account)

        ## Posting Progress UI ##
        self.posting_title = self.glade.get_widget("label220")
        self.posting_info = self.glade.get_widget("label218")
        self.posting_pgbar = self.glade.get_widget("progressbar2")
        self.posting_save_btn = self.glade.get_widget("button165")
        self.posting_save_btn.connect("clicked", self.on_saveas_post)         
        self.posting_try_btn = self.glade.get_widget("button166")
        self.posting_try_btn.connect("clicked", self.publish_post) 

        ## About UI ##
        self.about_ntbk = self.glade.get_widget("notebook12")
        self.about_ntbk.set_show_tabs(False)
        self.about_ntbk.connect("switch_page", self.about_ntbk_changed_page) 
        
        about_logo_img = self.glade.get_widget("image7")
        about_logo_img.set_from_file(imgdir + 'logo.png')

        self.credits_btn = self.glade.get_widget("button6")
        self.credits_btn.connect("clicked", self.show_page_notebook, \
                                                        self.about_ntbk, 1, None)  
        self.license_btn = self.glade.get_widget("button152")
        self.license_btn.connect("clicked", self.show_page_notebook, \
                                                        self.about_ntbk, 2, None) 
        self.about_desc_btn = self.glade.get_widget("button153")
        self.about_desc_btn.connect("clicked", self.show_page_notebook, \
                                                        self.about_ntbk, 0, None) 

        ## Set images for edition toolbar ##
        bold_img = self.glade.get_widget("bold_img")
        bold_img.set_from_file(imgdir + 'bold.png')
        italic_img = self.glade.get_widget("italic_img")
        italic_img.set_from_file(imgdir + 'italic.png')
        image_img = self.glade.get_widget("image_img")
        image_img.set_from_file(imgdir + 'image.png')
        link_img = self.glade.get_widget("link_img")
        link_img.set_from_file(imgdir + 'link.png')
        underline_img = self.glade.get_widget("underline_img")
        underline_img.set_from_file(imgdir + 'underline.png')
        strike_img = self.glade.get_widget("strike_img")
        strike_img.set_from_file(imgdir + 'strikethrough.png')
        parag_img = self.glade.get_widget("parag_img")
        parag_img.set_from_file(imgdir + 'paragraph.png')
        add_img = self.glade.get_widget("add_img")
        add_img.set_from_file(imgdir + 'add.png') 

        ## Images for more tags buttons ##
        left_img = self.glade.get_widget("left_img")
        left_img.set_from_file(imgdir + 'left.png') 
        right_img = self.glade.get_widget("right_img")
        right_img.set_from_file(imgdir + 'right.png')
        center_img = self.glade.get_widget("center_img")
        center_img.set_from_file(imgdir + 'center.png')
        fill_img = self.glade.get_widget("fill_img")
        fill_img.set_from_file(imgdir + 'fill.png')
        code_img = self.glade.get_widget("code_img")
        code_img.set_from_file(imgdir + 'code.png')
        quote_img = self.glade.get_widget("quote_img")
        quote_img.set_from_file(imgdir + 'quote.png')
        li_img = self.glade.get_widget("li_img")
        li_img.set_from_file(imgdir + 'li.png')
        ul_img = self.glade.get_widget("ul_img")
        ul_img.set_from_file(imgdir + 'ul.png')
        ol_img = self.glade.get_widget("ol_img")
        ol_img.set_from_file(imgdir + 'ol.png')
        fontcolor_img = self.glade.get_widget("fontcolor_img")
        fontcolor_img.set_from_file(imgdir + 'fontcolor.png')
        bgcolor_img = self.glade.get_widget("bgcolor_img")
        bgcolor_img.set_from_file(imgdir + 'bgcolor.png')
        fontsize_img = self.glade.get_widget("fontsize_img")
        fontsize_img.set_from_file(imgdir + 'fontsize.png')
        sup_img = self.glade.get_widget("sup_img")
        sup_img.set_from_file(imgdir + 'sup.png')
        sub_img = self.glade.get_widget("sub_img")
        sub_img.set_from_file(imgdir + 'sub.png')
        mono_img = self.glade.get_widget("mono_img")
        mono_img.set_from_file(imgdir + 'mono.png')
        cite_img = self.glade.get_widget("cite_img")
        cite_img.set_from_file(imgdir + 'cite.png')
        abbr_img = self.glade.get_widget("abbr_img")
        abbr_img.set_from_file(imgdir + 'abbr.png')
        acron_img = self.glade.get_widget("acron_img")
        acron_img.set_from_file(imgdir + 'acron.png')

        #print 'all ui: ', time.time() - start_time
        
        ## destroying the gtk glade window ##
        gtkwindow.destroy()

        self.window.show_all()

        #print 'showall: ', time.time() - start_time

        ## Creating the menu ##
        menu = gtk.Menu()
        self.window.set_menu(menu)
        new_menu = gtk.ImageMenuItem("New")
        new_img = gtk.Image()    
        new_img.set_from_file(imgdir + 'new.png')         
        new_menu.set_image(new_img) 
        new_menu.set_size_request(-1, 55)
        new_menu.connect("activate", self.on_new_post, 'menu')
        open_menu = gtk.ImageMenuItem("Open")
        open_img = gtk.Image()    
        open_img.set_from_file(imgdir + 'open.png')         
        open_menu.set_image(open_img)         
        open_menu.set_size_request(-1, 55)        
        open_menu.connect("activate", self.on_open_post)        
        save_menu = gtk.ImageMenuItem("Save")
        save_img = gtk.Image()    
        save_img.set_from_file(imgdir + 'save.png')
        save_menu.set_image(save_img)         
        save_menu.set_size_request(-1, 55)
        save_menu.connect("activate", self.on_save_post, '', '')
        saveas_menu = gtk.ImageMenuItem("Save As")
        saveas_img = gtk.Image()    
        saveas_img.set_from_file(imgdir + 'save.png')         
        saveas_menu.set_image(saveas_img)          
        saveas_menu.set_size_request(-1, 55)       
        saveas_menu.connect("activate", self.on_saveas_post)

        log_menu = gtk.ImageMenuItem("Log")
        log_img = gtk.Image()    
        log_img.set_from_file(imgdir + 'edit.png')         
        log_menu.set_image(log_img)         
        log_menu.set_size_request(-1, 55)        
        log_menu.connect("activate", self.on_show_log) 
                         
        menu.append(new_menu)
        menu.append(open_menu) 
        menu.append(save_menu)
        menu.append(saveas_menu)
        menu.append(log_menu)        
               
        menu.show_all()

        #print 'menu: ', time.time() - start_time

        try:
            self.set_font_to_widgets()
            self.set_initial_font_textview()
        except Exception, e:
            log = 'ERROR setting the font to the widgets:\n%s' % e
            io.set_log_to_file(logfile, log) 

        #print 'font: ', time.time() - start_time

    def quit_and_save(self, widget):
        fontsize = self.fontsizeNE.get_value()
        numpost = self.numpostsNE.get_value()
        perpage = self.perpageNE.get_value()
        picasa_login = self.picasa_login_entry.get_text()
        picasa_pass = self.picasa_pass_entry.get_text()
        
        albums = []
        for i in range(len(self.picasa_albumlist)):
            albums.append(self.picasa_albumlist[i][0])

        try:
            default_album = self.picasa_default_album_cb.get_active_text()
            albums.remove(default_album)
            albums.insert(0, default_album)
        except:
            print 'INFO: Not Picasa albums found.'            
        
        picasa_albums = ''
        for i in range(len(albums)):
            if i < len(albums)-1:
                picasa_albums = picasa_albums + albums[i] + '|||'
            else:
                picasa_albums = picasa_albums + albums[i]
        
        autosave = self.autosave_cb.get_active()
        as_time = self.astimeNE.get_value()
        as_path = self.autosave_path
        
        path_post  = self.path_post
        path_post_img = self.path_post_img
        path_add_img = self.path_add_img
        try:
            settings.save_settings(settings_db, fontsize, numpost, perpage, \
                            picasa_login, picasa_pass, picasa_albums, autosave, \
                            as_time, as_path, path_post, path_post_img, path_add_img)
        except Exception, e:
            print 'ERROR: Impossible save the general settings.'
            log = 'ERROR saving the general settings:\n%s' % e
            io.set_log_to_file(logfile, log) 
                                  
        #gtk.main_quit()
        loop.quit() 
        

    def on_show_log(self, widget):
        log = io.get_log_from_file(logfile)
        utils.show_log_dialog(log)

	#Functions for fullscreen
    def on_window_state_change(self, widget, event, *args):           
        if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
            self.window_in_fullscreen = True
        else:
            self.window_in_fullscreen = False

    #F6 fullscreen, F7 bigger font, F8 smaller font
    def on_key_press(self, widget, event, *args):
        page = self.main_ntbk.get_current_page()    
        if event.keyval == gtk.keysyms.F6:
            if self.window_in_fullscreen:
                self.window.unfullscreen ()
            else:
                self.window.fullscreen ()
        elif event.keyval == gtk.keysyms.F7:
            if (page == 1 or page == 2):
                if int(self.fontsize) >= 30:
                    utils.show_info_banner(widget, 'Maximum font size reached')
                    return
                else:
                    self.fontsize = str(int(self.fontsize) + 2)
                    self.txtPost.modify_font(pango.FontDescription("Arial " \
                                                                + self.fontsize))
                    self.txtPostExt.modify_font(pango.FontDescription("Arial " \
                                                                + self.fontsize))	
        elif event.keyval == gtk.keysyms.F8:
            if (page == 1 or page == 2):        
                if int(self.fontsize) <= 9:
                    utils.show_info_banner(widget, 'Minimal font size reached')
                    return
                else:
                    self.fontsize = str(int(self.fontsize) - 2)
                    self.txtPost.modify_font(pango.FontDescription("Arial " \
                                                                + self.fontsize))
                    self.txtPostExt.modify_font(pango.FontDescription("Arial "\
                                                                + self.fontsize))

    def show_page_notebook(self, widget, notebook, page, option):
        if option == 'on_tags':
            self.tags_ntbk.grab_focus()
            self.tags_ntbk.set_current_page(0)  
        elif option == 'flickr':
            self.managing_img = 'flickr'
            for image in all_images:
                if len(image) == 5:
                    image[4] = 1
                    image.append(0)
                    image.append(0)
                    image.append([])            
        elif option == 'picasa':
            self.managing_img = 'picasa'
            for image in all_images:            
                if len(image) == 8:
                    image.pop()
                    image.pop()
                    image.pop()
                    image[4] = self.picasa_default_album_cb.get_active_text()            
        elif option == 'media':
            self.media_ntbk.set_current_page(0)
        elif option == 'filters_img':
            self.color_bright_hbox.hide()
            self.colorize_btn.show()
            self.brightness_btn.show()
        elif option == 'edit_img':
            self.edit_img_ntbk.set_current_page(0)
        elif option == 'filters_img':
            self.color_bright_hbox.hide()
            self.colorize_btn.show()
            self.brightness_btn.show()
        elif option == 'add_link':
            self.textview_with_focus = self.grab_focus_to_textview()
            if self.textview_with_focus == None:
                utils.show_info_banner(widget, "Select the main or extended entry")
                return
            else:
                self.set_selected_text_to_common_entry(self.textview_with_focus)

            self.common_entry.set_text("http://")
            self.common_entry.select_region(7, 7)
            self.common_option = 'link'
            self.fontcolor_chooser.hide()
            self.label1_common.set_text("URL: ")
            self.label2_common.set_text("Text: ")
            self.hbox1_common.show()
            self.hbox2_common.show()  
            self.fontsize_hbox.hide()
        elif option == 'fontsize':
            self.set_selected_text_to_common_entry(self.textview_with_focus)        
            self.common_option = 'fontsize'    
            self.fontcolor_chooser.hide()    
            self.hbox1_common.hide()
            self.hbox2_common.hide()
            self.fontsize_hbox.show()
        elif option == 'fontcolor':            
            self.set_selected_text_to_common_entry(self.textview_with_focus)        
            self.common_option = 'fontcolor'    
            self.fontcolor_chooser.show()  
            self.label2_common.set_text("Text: ")
            self.hbox1_common.hide()
            self.hbox2_common.show()
            self.fontsize_hbox.hide()                        
        elif option == 'bgcolor':            
            self.set_selected_text_to_common_entry(self.textview_with_focus)        
            self.common_option = 'bgcolor'    
            self.fontcolor_chooser.show()
            self.label2_common.set_text("Text: ")    
            self.hbox1_common.hide()
            self.hbox2_common.show()
            self.fontsize_hbox.hide()                     
        elif option == 'add_abbr':
            self.set_selected_text_to_common_entry(self.textview_with_focus)        
            self.common_option = 'abbr'
            self.fontcolor_chooser.hide()
            self.common_entry.set_text("")
            self.label1_common.set_text("Text: ")
            self.label2_common.set_text("Abbreviation: ")
            self.hbox1_common.show()
            self.hbox2_common.show()  
            self.fontsize_hbox.hide()
        elif option == 'add_acronym':
            self.set_selected_text_to_common_entry(self.textview_with_focus)          
            self.common_option = 'acronym'
            self.fontcolor_chooser.hide()
            self.common_entry.set_text("")
            self.label1_common.set_text("Text: ")
            self.label2_common.set_text("Acronym: ")
            self.hbox1_common.show()
            self.hbox2_common.show()  
            self.fontsize_hbox.hide()      
        elif option == 'settings':
            self.settings_ntbk.set_current_page(0)
        elif option == 'media_settings':
            self.settings_ntbk.set_current_page(5)        
        elif option == 'new_account':
            #Clear the account entries and all.
            self.blogging_system_cbox.child.set_text('')
            self.account_name_entry.set_text('')
            self.wordpress_user_entry.set_text('')
            self.wordpress_pass_entry.set_text('')
            self.blogger_user_entry.set_text('')
            self.blogger_pass_entry.set_text('')            
            self.bloglist = []
            self.blogger_list.clear()
            self.blogger_blog_cbox.child.set_text('')
            self.blogurl_entry.set_text('')
            self.new_acc_wizard_ntbk.set_current_page(0)
            self.back_new_account_btn.set_sensitive(False)
            self.next_new_account_btn.set_sensitive(True)
            self.account_edited = False       
        elif option == 'show_pubopt':
            notebook.grab_focus()        
            self.main_ntbk.set_current_page(13)
        elif option == 'show_schedule':
            notebook.grab_focus()           
        elif option == 'manage_posts':
            self.manage_posts_ntbk.set_current_page(0) 
        elif option == 'preview':
            notebook.grab_focus()
            banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Previewing article')
            thread.start_new_thread(self.show_preview, (self.window, banner))
        elif option == 'cache_img':
            self.cache_model.clear()
            cached_images = self.GetCacheImages()
            self.cache_model = self.cache_images_model(cached_images)
            self.cache_tv.set_model(self.cache_model)
        elif option == 'back_add_img':
            notebook.grab_focus()
        elif option == 'html_img':
            try:
                self.set_html_tags_image_size()
            except:
                print 'no edit image'
            notebook.grab_focus()
        elif option == 'cancel_desc_img':
            notebook.grab_focus()
        elif option == 'adv_opt':
            notebook.grab_focus()
        elif option == 'home':
            notebook.grab_focus()
            recentposts = RecentPosts()
            recentposts_txt = recentposts.get()
            self.recentposts.set_markup(recentposts_txt)
            self.recentposts_title.set_markup('<b>Recent Posts:</b>')
        elif option == 'crop':
            self.do_ui_widgets_for_crop()
        elif option == 'resize_img':
            self.ResizeWidthNE.set_value(str(self.edit_pixbuf.get_width()))
            self.ResizeHeightNE.set_value(str(self.edit_pixbuf.get_height()))
        elif option == 'rotate_img':
            self.rotate_ntbk.set_current_page(0)
        elif option == 'rotate_angle':            
            self.AngleNE.set_value(0)
        elif option == 'about':
            self.about_ntbk.set_current_page(0)
        notebook.set_current_page(page)

    def about_ntbk_changed_page(self, widget, dummy, pagenum):
        if pagenum == 0:
            self.credits_btn.show()
            self.license_btn.show()
            self.about_desc_btn.hide()
        elif pagenum == 1:
            self.credits_btn.hide()
            self.license_btn.show()
            self.about_desc_btn.show()
        elif pagenum == 2:
            self.credits_btn.show()
            self.license_btn.hide()
            self.about_desc_btn. show()   
                    
    def on_notebook_changed(self, widget, dummy, pagenum):
        if pagenum == 4 or pagenum == 6 or pagenum == 7 or pagenum == 8 \
        or pagenum == 9:
            self.adv_opt_btn.hide()
            self.add_tags_btn.hide()
            self.preview_btn.hide()
            self.publish_btn.hide()
            self.main_btn.hide()
            self.main_hbox.show()
            self.edit_toolbar.hide()            
            if pagenum == 4:
                self.window.set_title('Maemo WordPy - Manage Posts')
                if self.blogsystem == 'Livejournal':
                    self.update_status_btn.set_sensitive(False)
                else: 
                    self.update_status_btn.set_sensitive(True)                
            elif pagenum == 6:
                self.window.set_title('Maemo WordPy - Manage Media')
            elif pagenum == 7:
                self.window.set_title('Maemo WordPy - Settings')
            elif pagenum == 8:
                self.window.set_title('Maemo WordPy - Help')
            elif pagenum == 9:
                self.window.set_title('Maemo WordPy - About')               

        if pagenum == 14:
            self.edit_toolbar.hide()
            self.add_tags_btn.hide()
            self.publish_btn.hide() 
            self.main_btn.hide()  
            self.preview_btn.hide()
            self.home_btn.set_sensitive(False)
            self.posting_save_btn.hide()    
            self.posting_try_btn.hide()
            self.adv_opt_btn.hide()     
            self.posting_pgbar.show()

        if pagenum == 10:
            self.adv_opt_btn.show()
            self.main_btn.show() 
            self.add_tags_btn.hide()            
            self.preview_btn.show()
            self.publish_btn.show()
            self.main_hbox.show()
            self.edit_toolbar.hide()         
            self.window.set_title('Maemo WordPy - Adding Tags')
            if self.blogsystem == 'Blogger' or self.blogsystem == 'Livejournal':
                self.cats_btn.set_sensitive(False)
            elif self.blogsystem == 'Wordpress':   
                self.cats_btn.set_sensitive(True)              
               
        if pagenum == 3 or pagenum == 11 or pagenum == 12:
            self.preview_btn.hide()
            self.adv_opt_btn.show()
            self.main_btn.show()
            self.main_hbox.show()
            self.edit_toolbar.hide()
            if pagenum == 3:
                self.window.set_title('Maemo WordPy - Previewing Post')
            else:           
                self.window.set_title('Maemo WordPy - Creating Post')
            

        elif (pagenum == 2 or pagenum == 13):
            self.adv_opt_btn.hide()
            self.preview_btn.show()
            self.main_btn.show()
            self.add_tags_btn.show()            
            self.main_hbox.show()
            if pagenum == 13:
                self.edit_toolbar.hide()
                if self.blogsystem == 'Blogger':
                    self.publishopt_combo.remove_text(3)
                    self.publishopt_combo.remove_text(2)
                    self.comments_cb.set_sensitive(False)
                    self.pings_cb.set_sensitive(False)
                    self.slug_entry.set_sensitive(False)
                    self.publishopt_combo.set_sensitive(True)
                    self.postpassword_entry.set_sensitive(False)
                    self.schedule_btn.set_sensitive(True)
                    self.publishopt_combo.get_model().clear()
                    self.publishopt_combo.insert_text(0, 'publish')                        
                    self.publishopt_combo.insert_text(1, 'draft') 
                    if (self.publishopt_combo.child.get_text() == 'private' \
                        or self.publishopt_combo.child.get_text() == 'pending'):
                        self.publishopt_combo.set_active(0)

                elif self.blogsystem == 'Livejournal':
                    self.comments_cb.set_sensitive(True)
                    self.pings_cb.set_sensitive(False)
                    self.slug_entry.set_sensitive(False)
                    self.postpassword_entry.set_sensitive(False)                    
                    self.publishopt_combo.set_sensitive(False) 
                    self.schedule_btn.set_sensitive(False)
                                                
                elif self.blogsystem == 'Wordpress':
                    self.comments_cb.set_sensitive(True)
                    self.pings_cb.set_sensitive(True)
                    self.slug_entry.set_sensitive(True)
                    self.publishopt_combo.set_sensitive(True) 
                    self.postpassword_entry.set_sensitive(True)
                    self.schedule_btn.set_sensitive(True)
                    self.publishopt_combo.get_model().clear()
                    self.publishopt_combo.insert_text(0, 'publish')                        
                    self.publishopt_combo.insert_text(1, 'draft')
                    self.publishopt_combo.insert_text(2, 'pending')
                    self.publishopt_combo.insert_text(3, 'private')                    
            else:
                self.edit_toolbar.show()
                if self.blogsystem == 'Blogger' or self.blogsystem == 'Livejournal':
                    self.txtPostExt.set_sensitive(False)
                    self.trackbacks_btn.set_sensitive(False)
                elif self.blogsystem == 'Wordpress':
                    self.txtPostExt.set_sensitive(True)     
                    self.trackbacks_btn.set_sensitive(True)
            self.window.set_title('Maemo WordPy - Creating Post')

        elif pagenum == 1:
            self.home_btn.show()
            self.adv_opt_btn.show()
            self.add_tags_btn.show()
            self.preview_btn.show()
            self.publish_btn.show()
            self.main_btn.hide()
            self.main_hbox.show()
            self.edit_toolbar.show()            
            self.window.set_title('Maemo WordPy - Creating Post')

        elif pagenum == 0:
            self.main_hbox.hide()
            self.edit_toolbar.hide()
            self.window.set_title('Maemo WordPy - Home')

    def set_initial_font_textview(self):
        self.txtPost.modify_font(pango.FontDescription("Arial "+ self.fontsize))	
        self.txtPostExt.modify_font(pango.FontDescription("Arial "+ self.fontsize))	

    def set_font_to_widgets(self):
        for label in range(1, 280):
            try:
                self.glade.get_widget('label' + \
                        str(label)).modify_font(pango.FontDescription("arial 16"))
            except AttributeError:
                pass        

        for button in range(1, 200):
            try:
                self.glade.get_widget('button' + \
                        str(button)).child.modify_font(pango.FontDescription("arial 16"))            
            except AttributeError:
                pass

        for entry in range(1, 50):
            try:
                self.glade.get_widget('entry' + \
                        str(entry)).modify_font(pango.FontDescription("arial 16"))            
            except AttributeError:
                pass        

        for combobox in range(1, 15):
            try:
                self.glade.get_widget('comboboxentry' + \
                    str(combobox)).child.modify_font(pango.FontDescription("arial 16"))            
            except AttributeError:
                pass 
                
        for checkbutton in range(1, 15):
            try: 
                self.glade.get_widget('checkbutton' + \
                str(checkbutton)).child.modify_font(pango.FontDescription("arial 16"))
            except AttributeError:
                pass         

        for radiobutton in range(1, 10):
            try: 
                self.glade.get_widget('radiobutton' + \
                str(radiobutton)).child.modify_font(pango.FontDescription("arial 16"))
            except AttributeError:
                pass

        for treeview in range(1, 12):
            try: 
                self.glade.get_widget('treeview' + \
                str(treeview)).modify_font(pango.FontDescription("arial 16"))
            except AttributeError:
                pass

    def show_more_tags_page(self, widget):
        self.textview_with_focus = self.grab_focus_to_textview()
        if self.textview_with_focus == None:
            utils.show_info_banner(widget, "Select the main or extended entry")
            return
        #We miss the focus from the textviews, so we need regrab the focus
        self.main_ntbk.set_current_page(11)
        self.textview_with_focus.grab_focus()

    def show_add_images_page(self, widget):
        self.textview_with_focus = self.grab_focus_to_textview()
        if self.textview_with_focus == None:
            utils.show_info_banner(widget, "Select the main or extended entry")
            return
        self.edit_toolbar.hide()            
        self.add_img_ntbk.set_current_page(0)
        self.main_ntbk.set_current_page(5)
        self.textview_with_focus.grab_focus()
        try:
            pixbuf = self.local_image_img.get_pixbuf()
            del pixbuf
        except:
            pass
        self.local_image_img.set_from_file(imgdir + 'img-missing.png')
        self.WidthNE.set_value(0)
        self.HeightNE.set_value(0)
        self.original_button.set_active(True)
        self.img_align.child.set_text('')
        self.en_alttext.set_text('')
        self.inet_img_entry.set_text('')
        self.local_img_label.set_text("")
        if self.blogsystem == 'Livejournal':
            self.local_btn.set_sensitive(False)
        else:
            self.local_btn.set_sensitive(True)           

    def on_cancel_insert_img(self, widget):
        #If cancel image dialog
        self.delete_temp_images()
        self.textview_with_focus.grab_focus()
        self.return_to_textview_window()

    def grab_focus_to_textview(self):
        if self.txtPost.is_focus():
            focused = self.txtPost
        elif self.txtPostExt.is_focus():
            focused = self.txtPostExt
        else:
            focused = None
            
        if focused is not None:
            focused.grab_focus()
            
        return focused  

    def insert_htmltags(self, widget, tag, back):
        from wordpy import htmltags
        try:
            self.textview_with_focus = self.grab_focus_to_textview()
            if self.textview_with_focus == None:
                utils.show_info_banner(widget, "Select the main or extended entry")
                return
            htmltags.insert_htmltag(self.textview_with_focus, tag)
            if back:
                self.return_to_textview_window()
        except Exception, e:
            log = 'ERROR inserting html tags in a post:\n%s' % e
            io.set_log_to_file(logfile, log) 

    def on_common_btn_action(self, widget):
        from wordpy import htmltags          
        if self.common_option == 'link':
            url = self.common_entry.get_text()
            text = self.common_text_entry.get_text()
            self.textview_with_focus.grab_focus()
            htmltags.insert_link_tag(self.textview_with_focus, url, text)
        elif self.common_option == 'abbr':
            abbr = self.common_entry.get_text()
            text = self.common_text_entry.get_text()
            self.textview_with_focus.grab_focus()
            htmltags.insert_abbr_or_acronym_tag(self.textview_with_focus, \
                                                'abbr', abbr, text)
        elif self.common_option == 'acronym':
            acronym = self.common_entry.get_text()
            text = self.common_text_entry.get_text()
            self.textview_with_focus.grab_focus()
            htmltags.insert_abbr_or_acronym_tag(self.textview_with_focus, \
                                                'acronym', acronym, text)
        elif self.common_option == 'fontsize':
            fontsize = self.fontsize_combo.get_active_text()
            text = self.common_text_entry.get_text()
            self.textview_with_focus.grab_focus()            
            htmltags.insert_fontsize(self.textview_with_focus, fontsize, text) 
        elif self.common_option == 'fontcolor':
            text = self.common_text_entry.get_text()
            self.textview_with_focus.grab_focus()
            htmltags.insert_bgcolor_or_fontcolor(self.textview_with_focus, \
                                    self.fontcolor_chooser, 'fontcolor', text)      
        elif self.common_option == 'bgcolor':
            text = self.common_text_entry.get_text()
            self.textview_with_focus.grab_focus()
            htmltags.insert_bgcolor_or_fontcolor(self.textview_with_focus, \
                                    self.fontcolor_chooser, 'bgcolor', text)            
        else:
            print 'no common action'

        #Returning to the previous window            
        self.return_to_textview_window()

    def on_cancel_common(self, widget):
        #FIXME: text should be the selected text in the textview, not the modified one.
        text = self.common_text_entry.get_text()
        self.textview_with_focus.grab_focus()
        utils.insert_text(self.textview_with_focus, text)
        self.return_to_textview_window()

    def set_selected_text_to_common_entry(self, textview):
        self.textview_with_focus.grab_focus()
        start, end = utils.get_selection_iters(self.textview_with_focus)
        self.common_text_entry.set_text(textview.get_buffer().get_text(start, end))
        textview.get_buffer().delete(start, end)
        
    def return_to_textview_window(self):       
        if self.txtPost.is_focus():
            self.main_ntbk.set_current_page(1)
        elif self.txtPostExt.is_focus():
            self.main_ntbk.set_current_page(2)

    ## Preview functions ##
    
    def show_preview(self, widget, banner):
        try:
            import gtkhtml2
            document = gtkhtml2.Document()
            document.connect('request-url', self.request_url)
            document.clear()
            document.open_stream('text/html')
            start, end = self.postBuffer.get_bounds()
            texto_buff = self.postBuffer.get_text(start, end)
			
            ext_start, ext_end = self.extendedBuffer.get_bounds()
            ext_texto_buff = self.extendedBuffer.get_text(ext_start, ext_end)
            if ext_texto_buff == "":
                pass
            else:
                texto_buff = texto_buff + '\n\n<a href="">[READ MORE]</a>\n\n' + ext_texto_buff
			
            html_final = texto_buff.replace("\n", "<br />")
            #CSS: <link type="text/css" rel="stylesheet" href="url"></link>
            header = '<html><head><meta http-equiv=content-type content="text/html; charset=UTF-8"></head>'
            title = "<h1>"+self.title_entry.get_text()+"</h1><br /><br />"
            body = "<body>"+title+html_final+"</body></html>"
            htmlpage = header+body
            document.write_stream(htmlpage)
            document.close_stream()
            self.htmlview.set_document(document)
            banner.destroy()
        except Exception, e:
            error_msg = 'Error previewing post'
            self.show_threaded_error(banner, widget, error_msg, e)        

    def request_url (self, doc, url, stream):
        from gnome import gnomevfs
        try:
            f = gnomevfs.open(url, gnomevfs.OPEN_READ)
        except Exception, e:
            log = 'ERROR opening url:%s\n%s' % (url, e)
            io.set_log_to_file(logfile, log)
            stream.close()
            return

        while 1:
            try:
                stream.write(f.read (1024))
            except gnomevfs.EOFError:
                break
        f.close()
        stream.close()

    def on_back_manage_posts(self, widget, notebook):
        notebook.set_current_page(0)

    def on_show_cats_ui(self, widget):
        self.add_cat_hbox.hide()
        self.add_cat_entry.hide()
        self.add_cat_btn.show()
        self.sync_cat_btn.show()
        #TODO: set offline categories
        self.tags_ntbk.set_current_page(2)
                       
    def on_show_tags_ui(self, widget):
        self.tags_ntbk.set_current_page(1)
        self.on_get_recenttags()        

    def on_set_tags(self, widget):
        self.tags_ntbk.set_current_page(0)
        self.main_ntbk.set_current_page(1)

    def on_unset_tags(self, widget):
        #self.tags_entry.set_text("")
        self.tags_ntbk.set_current_page(0)
        self.main_ntbk.set_current_page(1)
        
    def on_cancel_cats(self, widget):
        self.tags_ntbk.set_current_page(0)        
        self.main_ntbk.set_current_page(1)
        
    def on_show_add_cat_ui(self, widget):
        self.add_cat_hbox.show()
        self.add_cat_entry.show()        
        self.add_cat_btn.hide()
        self.sync_cat_btn.hide()

    def on_cancel_add_cat(self, widget):
        self.add_cat_hbox.hide()
        self.add_cat_entry.hide()        
        self.add_cat_btn.show()
        self.sync_cat_btn.show()

    def on_img_ntbk_changed(self, widget, dummy, pagenum):
        if pagenum == 0:
            self.cache_img_btn.show()
            self.back_imgntbk_btn.hide()
            self.html_img_btn.show()
            if self.blogsystem == 'Livejournal':
                self.local_btn.set_sensitive(False)
        elif pagenum == 5:
            self.html_img_btn.hide()
            self.back_imgntbk_btn.show()
            self.cache_img_btn.hide()
        else:
            self.back_imgntbk_btn.show()
            self.html_img_btn.show()
            self.cache_img_btn.hide()
            if pagenum == 1:
                if self.local_img_label.get_text().count("file://") > 0:
                    self.edit_img_btn.set_sensitive(True)
                else:
                    self.edit_img_btn.set_sensitive(False)
            elif pagenum == 6:
                from wordpy import imaged
                #self.edit_img.set_from_pixbuf(self.local_image_img.get_pixbuf())
                imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'edit', self.edit_img)           

    def show_color_bright_hbox(self, widget, show, option):
        if show:
            self.color_bright_hbox.show()
            self.colorizeBar.set_value(5)
            if option == 'brightness':
                self.colorize_btn.hide()
                self.brightness_btn.show()
            elif option == 'colorize':
                self.brightness_btn.hide()
                self.colorize_btn.show()
        else:
            self.color_bright_hbox.hide()
            self.colorize_btn.show()
            self.brightness_btn.show()

    def do_ui_widgets_for_crop(self):
        image_width, image_height = utils.set_thumbnail_size(self.edit_pixbuf, 'large')
        
        self.crop_draw.set_size_request(int(image_width), int(image_height))		
        crop_pixbuf = self.edit_pixbuf.scale_simple(int(image_width), int(image_height), gtk.gdk.INTERP_BILINEAR)
				
        x_spin = self.glade.get_widget("spinbutton4")
        x_spin.set_sensitive(False)
        width_spin = self.glade.get_widget("spinbutton6")
        width_spin.set_sensitive(False)
		
        x_adj = gtk.Adjustment(0, 0, self.edit_pixbuf.get_width(), 1, 10, 10)
        x_spin.set_adjustment(x_adj)
		
        width_adj = gtk.Adjustment(self.edit_pixbuf.get_width(), 1, self.edit_pixbuf.get_width(), 1, 10, 10)
        width_spin.set_adjustment(width_adj)
		
        y_spin = self.glade.get_widget("spinbutton5")
        y_spin.set_sensitive(False)
        height_spin =self.glade.get_widget("spinbutton7")
        height_spin.set_sensitive(False)
		
        y_adj = gtk.Adjustment(0, 0, self.edit_pixbuf.get_height(), 1, 10, 10)
        y_spin.set_adjustment(y_adj)
		
        height_adj = gtk.Adjustment(self.edit_pixbuf.get_height(), 1, self.edit_pixbuf.get_height(), 1, 10, 10)
        height_spin.set_adjustment(height_adj)
		
        try:
            width_spin.set_value(int(self.edit_pixbuf.get_width()))
            height_spin.set_value(int(self.edit_pixbuf.get_height()))
        except:
            print "Image size can't be obtained."
            
        self.crop_draw.connect("expose-event", self.crop_image_expose_cb, crop_pixbuf, image_width, image_height)
        self.crop_draw.connect("motion_notify_event", self.crop_image_mouse_moved, self.crop_draw, 0, 0, x_spin, y_spin, width_spin, height_spin, image_width, image_height, width_adj, height_adj)
        self.crop_draw.connect("button_press_event", self.crop_image_button_press, self.crop_draw)
        self.crop_draw.connect("button_release_event", self.crop_image_button_release)
		
        self.crop_draw.realize()
        self.crop_rectangle = [0, 0]
        self.drawing_crop_rectangle = False
        self.update_rectangle = False
        self.rect = None

    def crop_value_changed(self, currspinbox, x, y, width, height, width_adj, height_adj, image_width, image_height, image, type):
        if type == 0:   # X
            if x.get_value() + width.get_value() > self.edit_pixbuf.get_width():
                width.set_value(self.edit_pixbuf.get_width() - x.get_value())
        elif type == 1: # Y
            if y.get_value() + height.get_value() > self.edit_pixbuf.get_height():
                height.set_value(self.edit_pixbuf.get_height() - y.get_value())
        self.coords = [int(x.get_value()), int(y.get_value()), int(width.get_value()), int(height.get_value())]
        self.crop_rectangle[0] = int(round(float(self.coords[0])/self.edit_pixbuf.get_width()*image_width, 0))
        self.crop_rectangle[1] = int(round(float(self.coords[1])/self.edit_pixbuf.get_height()*image_height, 0))
        x2 = int(round(float(self.coords[2])/self.edit_pixbuf.get_width()*image_width, 0)) + self.crop_rectangle[0]
        y2 = int(round(float(self.coords[3])/self.edit_pixbuf.get_height()*image_height, 0)) + self.crop_rectangle[1]
        self.drawing_crop_rectangle = True
        self.update_rectangle = True
        self.crop_image_mouse_moved(None, None, image, x2, y2, x, y, width, height, image_width, image_height, width_adj, height_adj)
        self.update_rectangle = False
        self.drawing_crop_rectangle = False

    def crop_image_expose_cb(self, image, event, pixbuf, width, height):
        self.crop_draw.window.clear()
        image.window.draw_pixbuf(None, pixbuf, 0, 0, 0, 0, int(width), int(height))

    def crop_image_mouse_moved(self, widget, event, image, x2, y2, x, y, width, height, image_width, image_height, width_adj, height_adj):
        if event != None:
            x2, y2, state = event.window.get_pointer()
        if self.drawing_crop_rectangle:
            if self.crop_rectangle != None or self.update_rectangle:
                gc1 = image.window.new_gc(function=gtk.gdk.INVERT)
                if self.rect != None:
                    # Get rid of the previous drawn rectangle:
                    image.window.draw_rectangle(gc1, False, self.rect[0], self.rect[1], self.rect[2], self.rect[3])
                self.rect = [0, 0, 0, 0]
                if self.crop_rectangle[0] > x2:
                    self.rect[0] = x2
                    self.rect[2] = self.crop_rectangle[0]-x2
                else:
                    self.rect[0] = self.crop_rectangle[0]
                    self.rect[2] = x2-self.crop_rectangle[0]
                if self.crop_rectangle[1] > y2:
                    self.rect[1] = y2
                    self.rect[3] = self.crop_rectangle[1]-y2
                else:
                    self.rect[1] = self.crop_rectangle[1]
                    self.rect[3] = y2-self.crop_rectangle[1]
                image.window.draw_rectangle(gc1, False, self.rect[0], self.rect[1], self.rect[2], self.rect[3])
                # Convert the rectangle coordinates of the current image
                # to coordinates of pixbuf_original
                if self.rect[0] < 0:
                    self.rect[2] = self.rect[2] + self.rect[0]
                    self.rect[0] = 0
                if self.rect[1] < 0:
                    self.rect[3] = self.rect[3] + self.rect[1]
                    self.rect[1] = 0
                if event != None:
                    self.coords = [0,0,0,0]
                    self.coords[0] = int(round(float(self.rect[0])/image_width*self.edit_pixbuf.get_width(), 0))
                    self.coords[1] = int(round(float(self.rect[1])/image_height*self.edit_pixbuf.get_height(), 0))
                    self.coords[2] = int(round(float(self.rect[2])/image_width*self.edit_pixbuf.get_width(), 0))
                    self.coords[3] = int(round(float(self.rect[3])/image_height*self.edit_pixbuf.get_height(), 0))
                    if self.coords[0] + self.coords[2] > self.edit_pixbuf.get_width():
                        self.coords[2] = self.edit_pixbuf.get_width() - self.coords[0]
                    if self.coords[1] + self.coords[3] > self.edit_pixbuf.get_height():
                        self.coords[3] = self.edit_pixbuf.get_height() - self.coords[1]
                x.set_value(self.coords[0])
                y.set_value(self.coords[1])
                width.set_value(self.coords[2])
                height.set_value(self.coords[3])
                width_adj.set_property('upper', self.edit_pixbuf.get_width() - self.coords[0])
                height_adj.set_property('upper', self.edit_pixbuf.get_height() - self.coords[1])

    def crop_image_button_press(self, widget, event, image):
        x, y, state = event.window.get_pointer()
        if (state & gtk.gdk.BUTTON1_MASK):
            self.drawing_crop_rectangle = True
            self.crop_rectangle = [x, y]
            gc1 = image.window.new_gc(function=gtk.gdk.INVERT)
            if self.rect != None:
                # Get rid of the previous drawn rectangle:
                image.window.draw_rectangle(gc1, False, self.rect[0], self.rect[1], self.rect[2], self.rect[3])
                self.rect = None

    def crop_image_button_release(self, widget, event):
        x, y, state = event.window.get_pointer()
        if not (state & gtk.gdk.BUTTON1_MASK):
            self.drawing_crop_rectangle = False

    def new_account_wizard(self, widget, option):
        if option == 'back':
            if (self.blogging_system_cbox.get_active_text() == 'Wordpress' or \
                    self.blogging_system_cbox.get_active_text() == 'Livejournal'):
                #if self.new_acc_wizard_ntbk.get_current_page() == 3:
                #    self.new_acc_wizard_ntbk.set_current_page(1)
                #else:
                self.new_acc_wizard_ntbk.prev_page()
            elif self.blogging_system_cbox.get_active_text() == 'Blogger':
                if self.new_acc_wizard_ntbk.get_current_page() == 2:
                    self.new_acc_wizard_ntbk.set_current_page(0)
                else:
                    self.new_acc_wizard_ntbk.prev_page()
                
        elif option == 'next':
            if (self.blogging_system_cbox.get_active_text() == 'Wordpress' or \
                    self.blogging_system_cbox.get_active_text() == 'Livejournal'):
                if self.account_name_entry.get_text() == '':
                    utils.show_info_banner(widget, 'Please, add the account name')
                    return
                elif self.new_acc_wizard_ntbk.get_current_page() == 1:
                    #self.new_acc_wizard_ntbk.set_current_page(3)
                    pass
                else:
                    self.new_acc_wizard_ntbk.next_page()
            elif self.blogging_system_cbox.get_active_text() == 'Blogger':
                if self.account_name_entry.get_text() == '':
                    utils.show_info_banner(widget, 'Please, add the account name') 
                    return            
                if self.new_acc_wizard_ntbk.get_current_page() == 0:
                    self.new_acc_wizard_ntbk.set_current_page(2)
                else:
                    self.new_acc_wizard_ntbk.next_page()
            else:
                utils.show_info_banner(widget, 'Please, choose a blogging system') 

    def on_new_acc_wizard_ntbk_changed(self, widget, dummy, pagenum):
        if pagenum == 0:
            self.back_new_account_btn.set_sensitive(False)
            self.next_new_account_btn.set_sensitive(True)
        elif pagenum == 1:
            self.back_new_account_btn.set_sensitive(True)
            self.next_new_account_btn.set_sensitive(False)
            if self.blogging_system_cbox.get_active_text() == 'Livejournal':
                self.wordpress_url_hbox.hide()
            else:
                self.wordpress_url_hbox.show()                
        elif pagenum == 2:
            self.next_new_account_btn.set_sensitive(False)
            self.back_new_account_btn.set_sensitive(True)
        elif pagenum == 3:
            self.next_new_account_btn.set_sensitive(False) 
            self.back_new_account_btn.set_sensitive(True)
            self.show_acc_media_initial_ui(widget)       

    def on_autosave_cb_toggled(self, widget):
        if widget.get_active():
            self.as_time_hbox.set_sensitive(True)
            self.as_path_hbox.set_sensitive(True)
            self.autosave_on = 1
            self.stop_autosave_timer()
            self.autosave_timer = gobject.timeout_add(int(self.autosave_time*60000), \
                                                    self.check_autosave)            
        else:
            self.as_time_hbox.set_sensitive(False)
            self.as_path_hbox.set_sensitive(False)
            self.autosave_on = 0
            self.stop_autosave_timer()

    def create_accounts_list(self):
        lstore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_STRING)
        account_list = settings.get_account_list(accounts_db)
        ##this funcion will query to the accounts database and 
        #get the name and system of accounts 
        #account_list==[['name1', 'Blogger', 'True'], ['name2, 'Wordpress', 'False']]

        for item in account_list:
            lstore_iter = lstore.append()           
            if item[2] == 'True':
				lstore.set(lstore_iter, DEFAULTACCOUNT, '·', ACCOUNTNAME, \
				        item[ACCOUNTNAME], BLOGSYSTEM, item[BLOGSYSTEM])
            else:
                lstore.set(lstore_iter, DEFAULTACCOUNT, '', ACCOUNTNAME, item[ACCOUNTNAME], \
                        BLOGSYSTEM, item[BLOGSYSTEM])
        return lstore   

    def add_accounts_tv_columns(self, treeview):    
        # column for account name
        renderer = gtk.CellRendererText()
        renderer.set_property('scale', 2.5)
        column = gtk.TreeViewColumn('', renderer, text=DEFAULTACCOUNT)         
        treeview.append_column(column)    

        # column for account name
        renderer = gtk.CellRendererText()
        renderer.set_property('scale', 1.3)
        renderer.set_property('wrap-mode', gtk.WRAP_WORD)
        renderer.set_property('wrap-width', 400)         
        column = gtk.TreeViewColumn('Account Name', renderer, text=ACCOUNTNAME) 
        column.set_property("expand", True)           
        treeview.append_column(column)

        # column for blog system
        renderer = gtk.CellRendererText()
        renderer.set_property('scale', 1.3)    
        column = gtk.TreeViewColumn('Blogging system', renderer, text=BLOGSYSTEM)
        treeview.append_column(column)

    def finish_new_account(self, widget):
        blogsystem = self.blogging_system_cbox.get_active_text()
        accountname = self.account_name_entry.get_text()
        if blogsystem == 'Wordpress':
            user = self.wordpress_user_entry.get_text()
            password = self.wordpress_pass_entry.get_text()    
            blogid = 'None'
            blogname = 'None'
            blogurl = self.blogurl_entry.get_text()
        elif blogsystem == 'Livejournal':
            user = self.wordpress_user_entry.get_text()
            password = self.wordpress_pass_entry.get_text()    
            blogid = 'None'
            blogname = 'None'
            blogurl = 'None'
        elif blogsystem == 'Blogger':        
            user = self.blogger_user_entry.get_text()
            password = self.blogger_pass_entry.get_text()    
            try:
                blogname = self.blogger_blog_cbox.get_active_text()
                #print 'blogname: ', blogname
            except:
                blogname = 'None'
            try:
                blog_active = self.blogger_blog_cbox.get_active()
                blogid = self.bloglist[blog_active][1]
                #print 'blogid:', blogid
            except:
                blogid = 'None'
            blogurl = 'None'

        if self.account_edited:
            try:
                print 'edited account'
                oldname, oldsystem, siter, selmodel = self.get_name_and_system()
                settings.edit_account(accounts_db, accountname, blogsystem, user, \
                            password, blogid, blogname, blogurl, self.account_id, \
                            self.enabled, oldsystem, oldname)
                utils.show_info_banner(widget, 'Account edited')                     
                oldname, sel_iter, selmodel = utils.get_selected_from_treeview(self.accounts_tv, \
                                                    self.accounts_model, ACCOUNTNAME)

                #Enabled is as the last state, no setting here
                new_item = [accountname, blogsystem, '']                                           
                self.accounts_model.set(sel_iter, ACCOUNTNAME, new_item[ACCOUNTNAME], \
                                    BLOGSYSTEM, new_item[BLOGSYSTEM])   
                self.set_default_settings()
            except Exception, e:
                log = 'ERROR finishing to edit an account:\n%s' % e
                io.set_log_to_file(logfile, log)
        else: 
            try:
                name_list = []
                #Get all acountnames
                for i in range(len((self.accounts_model))):
                    name_list.append(self.accounts_model[i][0])
                #print name_list, accountname
                #If new accountname is duplicated, don't save
                if accountname in name_list:
                    utils.show_info_banner(widget, 'Duplicated account name')
                    return         

                print 'new account'
                n_tables = settings.new_account(accounts_db, accountname, blogsystem, \
                                    user, password, blogid, blogname, blogurl)

                if n_tables == 0:
                    print 'enabling first account'
                    enabled = '·'
                    self.set_default_settings()
                else:
                    enabled = ''
                new_item = [accountname, blogsystem, enabled]
                newiter = self.accounts_model.append()
                self.accounts_model.set(newiter, DEFAULTACCOUNT, new_item[DEFAULTACCOUNT], \
                    ACCOUNTNAME, new_item[ACCOUNTNAME], BLOGSYSTEM, new_item[BLOGSYSTEM])

                self.set_default_settings()

                utils.show_info_banner(widget, 'Account added')
            except Exception, e:
                log = 'ERROR finishing to create an account:\n%s' % e
                io.set_log_to_file(logfile, log)

        self.settings_ntbk.set_current_page(1)
        self.account_edited = False


    def edit_account(self, widget):
        try:
            if not utils.is_treeview_selected(self.accounts_tv):
                return
        
            self.account_edited = True
            accountname, blogsystem, siter, selmodel = self.get_name_and_system()
                                                    
            account = settings.get_account_settings(accounts_db, blogsystem, accountname)
            ##account = (id, accountname, user, password, blogid, blogname, blogurl, enabled)

            self.account_id = account[0]    
            self.blogging_system_cbox.child.set_text(blogsystem)
            self.account_name_entry.set_text(account[1])
            if (blogsystem == 'Wordpress' or blogsystem == 'Livejournal'):
                self.wordpress_user_entry.set_text(account[2])
                self.wordpress_pass_entry.set_text(account[3])
            elif blogsystem == 'Blogger':
                self.blogger_user_entry.set_text(account[2])
                self.blogger_pass_entry.set_text(account[3])

            blogid = account[4]
            blogname = account[5]        
            self.bloglist = []
            self.bloglist.append([blogname, blogid])
            self.blogger_list.clear()
            self.blogger_list.append((blogname,))
            self.blogger_blog_cbox.set_active(0)         
            
            self.blogurl_entry.set_text(account[6])
            self.enabled = account[7]
            #print 'enabled: ', self.enabled

            self.settings_ntbk.set_current_page(3)
            self.new_acc_wizard_ntbk.set_current_page(0)
            self.back_new_account_btn.set_sensitive(False)
            self.next_new_account_btn.set_sensitive(True)
        except Exception, e:
            log = 'ERROR editing account:\n%s' % e
            io.set_log_to_file(logfile, log)


    def get_name_and_system(self):
        name, seliter, selmodel = utils.get_selected_from_treeview(self.accounts_tv, \
                                            self.accounts_model, ACCOUNTNAME)        
        system, seliter, selmodel = utils.get_selected_from_treeview(self.accounts_tv, \
                                            self.accounts_model, BLOGSYSTEM)      
        return name, system, seliter, selmodel

    def delete_account(self, widget):
        try:
            if not utils.is_treeview_selected(self.accounts_tv):
                return    
           
            accountname, blogsystem, seliter, selmodel = self.get_name_and_system()

            for i in range(len((selmodel))):
                if selmodel[i][0] == accountname:
                    row_selected = i

            confirmation = utils.show_confirmation(self.window, \
                                    'Are you sure?\nIt will delete the account') 
            if confirmation == True:
                if selmodel[row_selected][2] == '·':
                    print 'is default account'
                    selmodel.remove(seliter)
                    if len(self.accounts_model) > 0:
                        self.accounts_model[0][2] = '·'
                        newblogsystem = self.accounts_model[0][1]
                        newaccountname = self.accounts_model[0][0]
                        settings.make_default(accounts_db, newblogsystem, \
                                                        newaccountname, True)
                else:
                    print 'isnt default account'
                    selmodel.remove(seliter)           
                  
                settings.delete_account(accounts_db, blogsystem, accountname)
                self.set_default_settings()           
                utils.show_info_banner(widget, 'Account deleted')
     
            else:
                return  
        except Exception, e:
            log = 'ERROR deleting account:\n%s' % e
            io.set_log_to_file(logfile, log)
        
    def make_default_account(self, widget):
        try:
            if not utils.is_treeview_selected(self.accounts_tv):
                return 

            print 'defaulting'
            accountname, blogsystem, seliter, selmodel = self.get_name_and_system()
            
            #[['account1', 'Blogger', 'True'], ['account2', 'Wordpress', 'False']]
            accountlist = settings.get_account_list(accounts_db)
            for i in range(len(accountlist)):
                if accountlist[i][2] == 'True':
                    oldname = accountlist[i][0]
                    oldsystem = accountlist[i][1]

            try:
                settings.make_default(accounts_db, oldsystem, oldname, False)
            except UnboundLocalError:
                pass
                
            settings.make_default(accounts_db, blogsystem, accountname, True)      
            
            # Unsetting the old default account in the treeview.
            for i in range(len(self.accounts_model)):
                if oldname == self.accounts_model[i][0]:
                    self.accounts_model[i][2] = ''     
                          
            new_item = [accountname, blogsystem, '·']
            self.accounts_model.set(seliter,  DEFAULTACCOUNT, new_item[DEFAULTACCOUNT], \
                    ACCOUNTNAME, new_item[ACCOUNTNAME], BLOGSYSTEM, new_item[BLOGSYSTEM])

            self.set_default_settings()
            self.manage_posts_model.clear()
        except Exception, e:
            log = 'ERROR marking the default account:\n%s' % e
            io.set_log_to_file(logfile, log)

    def set_default_settings(self):
        default_account, def_settings = settings.get_default_settings(accounts_db, \
                                                                settings_db)
        
        #print 'set: ', def_settings
        
        try:
            self.accountname = default_account[0][0]
            self.blogsystem = default_account[0][1]
            self.user = default_account[0][2]
            self.password = default_account[0][3]
            self.blogid = default_account[0][4]
            self.blogname = default_account[0][5]
            self.url = default_account[0][6]
        except:
            log = "INFO: There isn't defined accounts"
            io.set_log_to_file(logfile, log)
            self.accountname = ''
            self.blogsystem = ''
            self.user = ''
            self.password = ''
            self.blogid = ''
            self.blogname = ''
            self.url = '' 

        try:
            self.fontsize = str(def_settings[0][1])
            self.numposts = int(def_settings[0][2])
            self.perpage = int(def_settings[0][3])
            self.picasa_login = def_settings[0][4]
            self.picasa_pass = def_settings[0][5]
            self.picasa_albums = []
            #print def_settings[0][6].split('|||')
            for i in def_settings[0][6].split('|||'):
                self.picasa_albums.append(i)
            self.autosave_on = int(def_settings[0][7])
            self.autosave_time = int(def_settings[0][8])
            self.autosave_path = def_settings[0][9]
            if self.autosave_path == '':
                self.autosave_path = '/home/user/MyDocs/.documents'
            self.path_post = def_settings[0][10]
            self.path_post_img = def_settings[0][11]
            self.path_add_img = def_settings[0][12]
        except:
            log = "INFO: There isn't defined settings"
            io.set_log_to_file(logfile, log)
            self.fontsize = '16'
            self.numposts = 25
            self.perpage = 10
            self.picasa_login = ''
            self.picasa_pass = ''
            self.picasa_albums = []
            self.autosave_on = False
            self.autosave_time = 5
            self.autosave_path = '/home/user/MyDocs/.documents'
            self.path_post = '/home/user/MyDocs/.documents'
            self.path_post_img = '/home/user/MyDocs/.images'
            self.path_add_img = '/home/user/MyDocs/.images'

    def on_setting_change(self, widget, setting):
        if setting == 'fontsize':
            self.fontsize = str(self.fontsizeNE.get_value())
            self.txtPost.modify_font(pango.FontDescription("Arial "+self.fontsize))
            self.txtPostExt.modify_font(pango.FontDescription("Arial "+self.fontsize))
        elif setting == 'astime':
            self.autosave_time = int(self.astimeNE.get_value())
            self.stop_autosave_timer()
            self.autosave_timer = gobject.timeout_add (int(self.autosave_time*60000), \
                                                        self.check_autosave)
        elif setting == 'numposts':
            self.numposts = int(self.numpostsNE.get_value())
        elif setting == 'picasa_login': 
            self.picasa_login = self.picasa_login_entry.get_text()
        elif setting == 'picasa_pass':
           self.picasa_pass = self.picasa_pass_entry.get_text()
        elif setting == 'perpage':
            self.perpage = int(self.perpageNE.get_value())

    def get_blogger_bloglist(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', 'Getting blogs')
        thread.start_new(self._do_get_blogger_bloglist, (banner, widget)) 

    def _do_get_blogger_bloglist(self, banner, widget):
        from wordpy import blogger
        try:
            self.blogger_list.clear()        
            user = self.blogger_user_entry.get_text()
            password = self.blogger_pass_entry.get_text()
            bgclient = blogger.create_blogger_client(user, password)
            #bloglist = [['blog1', 12], ['blog2', 45]]
            self.bloglist = blogger.get_blog_list(bgclient)

            #print 'bloglist:', self.bloglist
            for item in self.bloglist:
                self.blogger_list.append((item[0],))
            self.blogger_blog_cbox.set_active(0)
            banner.destroy()
        except Exception, e:
            error_msg = 'Error getting the blog list'
            self.show_threaded_error(banner, widget, error_msg, e)
            
    def on_new_post(self, widget, option):
        #showing confirmation dialog
        if option == 'menu':
            confirmation = utils.show_confirmation(self.window, \
                            'Are you sure?\nIt will delete the current entry')
        else:
            confirmation = True

        if confirmation == True:
            ## Clear all text, title, tags, etc.
            self.title_entry.set_text("")
            self.postBuffer.delete(self.postBuffer.get_start_iter(), \
                                    self.postBuffer.get_end_iter())
            self.extendedBuffer.delete(self.extendedBuffer.get_start_iter(), \
                                    self.extendedBuffer.get_end_iter())
            self.tbacks_entry.set_text("")
            self.techtags_entry.set_text("")
            self.tags_entry.set_text("")
            self.postpassword_entry.set_text("")
            self.slug_entry.set_text("")
            self.pings_cb.set_active(True)
            self.comments_cb.set_active(True)
            self.publishopt_combo.set_active(0)
            self.schedule_widgets_to_actual_time()
            self.cats_model.clear()
            self.is_edited = False
            self.post_edited_btn.hide()
        else:
            return

    def on_open_post(self, widget):
        try:
            self.xml_file = utils.file_browser(self.window, gtk.FILE_CHOOSER_ACTION_OPEN, \
                                "Open Post", self.path_post, "", "", False)

            if (self.xml_file == False) or (self.xml_file== None):
                print 'No file selected.'
                return
            else:
                self.path_post = os.path.dirname(self.xml_file)       
                post_loaded = io.xml_load_from_file(self.xml_file, widget, utils)
                self.set_post_loaded(post_loaded)
        except Exception, e:
            log = 'ERROR opening the post:\n%s' % e
            io.set_log_to_file(logfile, log)
            
    def set_post_loaded(self, post_loaded):
        #post_loaded = 
        # (title, post, excerpt, extended, technorati, trackbacks, tags, \
        # password, slug)
        self.title_entry.set_text(post_loaded[0])
        self.postBuffer.set_text(post_loaded[1])
        #self.excerpt.set_text(post_loaded[2])
        self.extendedBuffer.set_text(post_loaded[3])
        self.techtags_entry.set_text(post_loaded[4])        
        self.tbacks_entry.set_text(post_loaded[5])
        self.tags_entry.set_text(post_loaded[6])
        self.postpassword_entry.set_text(post_loaded[7])
        self.slug_entry.set_text(post_loaded[8])
        if post_loaded[9] == '':
            self.publishopt_combo.child.set_text('publish')        
        else:
            self.publishopt_combo.child.set_text(post_loaded[9])
        try:
            date, hour, minutes = post_loaded[10].split('||')
            self.date_schedule_entry.set_text(date)
            self.hoursNE.set_value(hour)
            self.minutesNE.set_value(minutes)
            self.schedule_cbtn.set_active(False)
        except:
            self.schedule_widgets_to_actual_time()
        self.main_ntbk.set_current_page(1)
        self.is_edited = False
        self.post_edited_btn.hide()

    def schedule_widgets_to_actual_time(self):
        import time
        date = time.localtime()
        self.date_schedule_entry.set_text('%s/%s/%s' % (date[2], date[1], date[0]))
        self.hoursNE.set_value(date[3])
        self.minutesNE.set_value(date[4]) 
        self.schedule_cbtn.set_active(False)

    def get_post_options(self):
        title = self.title_entry.get_text()
        start, end = self.postBuffer.get_bounds()
        text = self.postBuffer.get_text(start, end)
        excerpt = ''        
        start, end = self.extendedBuffer.get_bounds()
        extended = self.extendedBuffer.get_text(start, end)
        ttags = self.techtags_entry.get_text()
        tbacks = self.tbacks_entry.get_text()
        tags = self.tags_entry.get_text()   
        password = self.postpassword_entry.get_text()
        slug = self.slug_entry.get_text()     
        status = self.publishopt_combo.get_active_text()
        date = self.date_schedule_entry.get_text()
        hour = self.hoursNE.get_value()
        minutes = self.minutesNE.get_value()
        date_time = '%s||%s||%s' % (date, hour, minutes)
        post = (title, text, excerpt, extended, ttags, tbacks, tags, password, \
                slug, status, date_time)
        
        return post

    def on_save_post(self, widget, filename, option):
        try:
            if option == 'autosave':
                self.xml_file = filename
            else:   
                if (self.xml_file == None) or (self.xml_file == False):
                    self.xml_file = utils.file_browser(self.window, gtk.FILE_CHOOSER_ACTION_SAVE, \
                                "Save Post", self.path_post, "", "wpx", False)         

            if (self.xml_file):
                self.path_post = os.path.dirname(self.xml_file) 
                post = self.get_post_options()
                io.xml_save_to_file(self.xml_file, widget, utils, post)
                if option != 'autosave':
                    utils.show_info_banner(widget, 'Post saved')
        except Exception, e:
            log = 'ERROR saving the post:\n%s' % e
            io.set_log_to_file(logfile, log)
        
    def on_saveas_post(self, widget):
        try:
            xml_file = "Untitled"        
            if (self.xml_file != None) and (self.xml_file != False):
                xml_file = os.path.basename(self.xml_file)

            xml_file = utils.file_browser(self.window, gtk.FILE_CHOOSER_ACTION_SAVE, \
                                "Save Post", self.path_post, xml_file, "wpx", False)
            if (xml_file):
                post = self.get_post_options()
                if (io.xml_save_to_file(xml_file, widget, utils, post)):
                    self.xml_file = xml_file
                    self.path_post = os.path.dirname(self.xml_file)
                    utils.show_info_banner(widget, 'Post saved')
        except Exception, e:
            log = 'ERROR saving as... the post:\n%s' % e
            io.set_log_to_file(logfile, log)

    def stop_autosave_timer(self):
        if self.autosave_timer > 0:
            gobject.source_remove(self.autosave_timer)
        self.autosave_timer = 0

    def check_autosave(self):
        title = self.title_entry.get_text()
        if title == '':
            import time
            date = time.localtime()
            date = '%s-%s-%s-' % (date[2], date[1], date[0])        
            title = date + 'autosaved_post'
        #print 'checking', self.autosave_on
        if self.autosave_on == 1:
            try:
                filename = self.autosave_path + '/%s.wpx' % title
                self.on_save_post(self.window, filename, 'autosave')
                self.autosave_timer = gobject.timeout_add (int(self.autosave_time*60000), \
                                                        self.check_autosave)
            except Exception, e:
                log = 'ERROR autosaving post:\n%s' % e
                io.set_log_to_file(logfile, log)
        else:
            self.stop_autosave_timer()

    def on_select_autosave_path(self, widget):
        dirname = utils.file_browser(self.window, \
                                    gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, \
                                    'Select autosave path', \
                                    '/home/user/MyDocs/.documents', '', '', False)

        if dirname:
            self.as_path_entry.set_text(dirname)
            self.autosave_path = dirname

    # Recent Tags #
    def return_recenttags_list(self):
        taglist = []    
        try:
            rt = RecentTags()
            recenttags = rt.get()       
            for i in recenttags:
                taglist.append([False, i])
                
            return taglist
        except Exception, e:
            log = 'ERROR getting the recent tags list:\n%s' % e
            io.set_log_to_file(logfile, log)
            return taglist 
                  
    def on_get_recenttags(self):
        self.recenttags_model.clear()        
        recenttags = self.return_recenttags_list()
        self.recenttags_model = self.__create_recenttags_model(recenttags) 
        self.recenttags_tv.set_model(self.recenttags_model) 
        
    def _recenttags_tv_columns(self, treeview):
        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.recenttags_toggled)
        column = gtk.TreeViewColumn('', renderer, active=0)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_fixed_width(50)
        treeview.append_column(column)

        column = gtk.TreeViewColumn('', gtk.CellRendererText(), text=1)       
        treeview.append_column(column)        

    def recenttags_toggled(self, cell, path):
        seliter = self.recenttags_model.get_iter((int(path),))
        fixed = self.recenttags_model.get_value(seliter, 0)

        fixed = not fixed
        self.recenttags_model.set(seliter, 0, fixed)

    def __create_recenttags_model(self, recenttags):
        lstore = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
        for item in recenttags:
            iter = lstore.append()
            lstore.set(iter, 0, item[0], 1, item[1])
        return lstore 

    def on_set_recenttags_to_post(self):
        tags = ''
        for i in range(len(self.recenttags_model)):
            if self.recenttags_model[i][0] == True:
                tags = tags + self.recenttags_model[i][1] + ', '

        if tags != '':
            tags = tags[:-2]

        return tags

    # Schedule posts #
    def on_calendar_selected(self, widget):
        day = str(widget.get_property("day"))
        month = str(widget.get_property("month")+1)
        year = str(widget.get_property("year"))
        self.date_schedule_entry.set_text(day+'/'+month+'/'+year)

    # Manage Wordpress Categories #
    def add_cat_to_server(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Creating category')
        thread.start_new(self._do_add_cat_to_server, (banner, widget))    
    
    def _do_add_cat_to_server(self, banner, widget):
        from wordpy import wordpress
        try:
            cat = wordpress.WordPressNewCategory()
            cat.name = self.add_cat_entry.get_text()
            wpclient = wordpress.create_wordpress_client(self.url, self.user, \
                                                        self.password) 
            if wpclient is not None:
                wpclient.newCategory(cat)
                self.on_cancel_add_cat(widget)
                banner.destroy()
            else:
                e = 'Wordpress authentication not created'
                error_msg = 'Error creating new category'
                self.show_threaded_error(banner, widget, error_msg, e) 
                banner.destroy()                
        except Exception, e:
            error_msg = 'Error creating new category'
            self.show_threaded_error(banner, widget, error_msg, e)


    def on_get_cats(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Getting categories')
        thread.start_new(self._do_on_get_cats, (banner, widget))  

    def _do_on_get_cats(self, banner, widget):
        self.cats_model.clear()
        cats = self.return_category_list(banner, widget)
        self.cats_model = self.__create_cats_model(cats) 
        self.categories_tv.set_model(self.cats_model) 
        
    def _cats_tv_columns(self, treeview):
        renderer = gtk.CellRendererToggle()        
        renderer.connect('toggled', self.cats_toggled)
        column = gtk.TreeViewColumn('', renderer, active=0)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_fixed_width(50)
        treeview.append_column(column)

        column = gtk.TreeViewColumn('ID', gtk.CellRendererText(), text=1)
        column.set_visible(False)
        treeview.append_column(column)

        column = gtk.TreeViewColumn('Offline Categories', gtk.CellRendererText(), text=2)
        treeview.append_column(column)        

    def cats_toggled(self, cell, path):
        seliter = self.cats_model.get_iter((int(path),))
        fixed = self.cats_model.get_value(seliter, 0)

        fixed = not fixed
        self.cats_model.set(seliter, 0, fixed)

    def __create_cats_model(self, cats):
        lstore = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, \
                                gobject.TYPE_STRING)
        #Added the category list to lstore
        for item in cats:
            iter = lstore.append()
            #print item, item[0]
            lstore.set(iter, 0, item[0], 1, item[1], 2, item[2])
        return lstore 

    def return_category_list(self, banner, widget):
        from wordpy import wordpress
        cats = []        
        try:
            wpclient = wordpress.create_wordpress_client(self.url, self.user, \
                                                            self.password)            
            if wpclient is None:
                e = 'Wordpress authentication not created'
                error_msg = 'Error getting categories'
                self.show_threaded_error(banner, widget, error_msg, e) 
                banner.destroy()
            else:
                wpclient.selectBlog(0)
                categories = tuple(wpclient.getCategoryList())
                for i in categories:
                    cats.append([False, i.id, i.name])
                    #print i.name, i.id
                ##Sort categories alphabetically (apparently it's sorted automatically)
                #cats.sort(lambda x, y: cmp(x[2],y[2])) 
                banner.destroy() 
            return cats
        except Exception, e:
            error_msg = 'Error getting categories'
            self.show_threaded_error(banner, widget, error_msg, e)
            return cats

    def on_set_cats_to_post(self, widget):
        for i in range(len(self.cats_model)):
            if self.cats_model[i][0] == True:
                #print self.cats_model[i][2]
                pass
                
        self.main_ntbk.set_current_page(1)
              
    ## Manage Posts Functions ##

    def on_clear_manage_posts(self, clicked):
        self.manage_posts_model.clear()

    def __create_manage_posts_model(self, posts):
        lstore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        for item in posts:
            iter = lstore.append()
            lstore.set(iter, 0, item[0], 1, item[1], 2, item[2])

        return lstore 
        
    def _manage_posts_tv_columns(self, treeview):
        renderer = gtk.CellRendererText()
        renderer.set_property('wrap-mode', gtk.WRAP_WORD)
        renderer.set_property('wrap-width', 400)
        renderer.set_property('scale', 1.2)        
        column = gtk.TreeViewColumn('Title', renderer, text=0)
        column.set_expand(True)
        treeview.append_column(column)

        column = gtk.TreeViewColumn('Id', gtk.CellRendererText(), text=1)
        column.set_visible(False)
        treeview.append_column(column)

        column = gtk.TreeViewColumn("Status", gtk.CellRendererText(), text=2)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_fixed_width(150)             
        treeview.append_column(column)  

    def on_get_posts_to_manage(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Getting posts')
        thread.start_new(self._do_get_posts_to_manage, (banner, widget))
    
    def _do_get_posts_to_manage(self, banner, widget):
        try:
            self.manage_posts_model.clear()
            if self.blogsystem == 'Wordpress':
                from wordpy import wordpress
                posts = wordpress.get_posts_with_status(self.url, self.user, \
                                                self.password, self.numposts)
            elif self.blogsystem == 'Blogger':
                from wordpy import blogger
                posts = blogger.get_posts_with_status(self.user, self.password, \
                                                    self.blogid, self.numposts)
            elif self.blogsystem == 'Livejournal':            
                from wordpy import livejournal
                posts = livejournal.post_list_formatted(self.user, self.password, \
                                                        self.numposts)
                
            #posts = [['title', '283732', "publish"]]
            self.manage_posts_model = self.__create_manage_posts_model(posts)
            self.manage_posts_tv.set_model(self.manage_posts_model)
            banner.destroy()
        except Exception, e:       
            error_msg = 'Error getting the post list'
            self.show_threaded_error(banner, widget, error_msg, e)

    def on_delete_post(self, widget):
        confirmation = utils.show_confirmation(self.window, \
                        'Are you sure?\nIt will delete the post from the server')     

        if confirmation:
            banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Deleting post')
            thread.start_new(self._do_delete_post, (banner, widget))    
    
    def _do_delete_post(self, banner, widget):
        try:
            selected = utils.is_treeview_selected(self.manage_posts_tv)
            if not selected:
                banner.destroy()
                return                  
            #self.activate_led()
            
            postid, seliter, selmodel = utils.get_selected_from_treeview(self.manage_posts_tv, \
                                                                self.manage_posts_model, 1)
            
            if self.blogsystem == 'Wordpress':
                from wordpy import wordpress
                wpclient = wordpress.create_wordpress_client(self.url, self.user, \
                                                            self.password)        
            elif self.blogsystem == 'Blogger':
                from wordpy import blogger
                bgclient = blogger.create_blogger_client(self.user, self.password)
            elif self.blogsystem == 'Livejournal':
                from wordpy import livejournal
                ljclient = livejournal.create_livejournal_client(self.user, \
                                                                self.password)             
            
            if self.blogsystem == 'Wordpress':
                wpclient.deletePost(postid)
            elif self.blogsystem == 'Blogger':
                bgclient.Delete(postid)
            elif self.blogsystem == 'Livejournal':
                ljclient.del_event(postid)
                            
            self.manage_posts_model.remove(seliter)
            banner.destroy()
            #self.stop_led()
        except Exception, e:
            error_msg = 'Error deleting posts'
            self.show_threaded_error(banner, widget, error_msg, e)

    def on_edit_post(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Getting post to edit')
        thread.start_new(self._do_edit_post, (banner, widget))    
    
    def _do_edit_post(self, banner, widget):
        try:
            selected = utils.is_treeview_selected(self.manage_posts_tv)
            if not selected:
                banner.destroy()
                return                   
                
            postid, seliter, selmodel = utils.get_selected_from_treeview(self.manage_posts_tv, \
                                                                self.manage_posts_model, 1)
            if self.blogsystem == 'Wordpress':
                from wordpy import wordpress
                post = wordpress.get_post_filtered(self.url, self.user, self.password, postid)
            elif self.blogsystem == 'Blogger':
                from wordpy import blogger
                post = blogger.get_post_filtered(self.user, self.password, postid, self.blogid)
            elif self.blogsystem == 'Livejournal':
                from wordpy import livejournal
                post = livejournal.get_post_filtered(self.user, self.password, postid)

            #print post
            self.set_edited_post(post)
            banner.destroy()
        except Exception, e:
            error_msg = 'Error getting the post to edit'
            self.show_threaded_error(banner, widget, error_msg, e)

    def set_edited_post(self, post):
        ## post = (title, content, extended, excerpt, ttags, tracks, tags, cats, 
        ## date, slug, pssword, comments, pings, status)
        self.title_entry.set_text(post[0])
        self.postBuffer.set_text(post[1])
        self.extendedBuffer.set_text(post[2])
        #self.excerpt.set_text(post[3])
        self.techtags_entry.set_text(post[4])
        self.tbacks_entry.set_text(post[5])        
        self.tags_entry.set_text(post[6])
        self.cats_model.clear()
        self.cats_model = self.__create_cats_model(post[7])
        self.categories_tv.set_model(self.cats_model)
        
        self.date = post[8]
        #print 'date:', self.date
        #wordpress = (2008, 9, 24, 13, 56, 29, 2, 268, -1)
        #blogger = 2008-09-15T11:11:00.001-07:00
        #livejournal = 2008-09-26 21:00:00
        if self.blogsystem == 'Wordpress' or self.blogsystem == 'Blogger':
            try:
                self.date_schedule_entry.set_text('%s/%s/%s' % (post[8][2],post[8][1],post[8][0]))
                self.hoursNE.set_value(post[8][3])
                self.minutesNE.set_value(post[8][4])
            except:
                self.schedule_widgets_to_actual_time()
        
        self.slug_entry.set_text(post[9])
        self.postpassword_entry.set_text(post[10])
        try:
            self.comments_cb.set_active(post[11])
            self.pings_cb.set_active(post[12])
        except:
            pass
        self.publishopt_combo.child.set_text(post[13])
        self.postid = post[14]
        self.main_ntbk.set_current_page(1)
        self.is_edited = True
        self.post_edited_btn.show()

    def show_is_edited_post(self, widget):
        utils.show_info_banner(widget, "This is an edited post")

    def on_show_change_status_ui(self, widget): 
        if not utils.is_treeview_selected(self.manage_posts_tv):
            return
        
        status, seliter, selmodel = utils.get_selected_from_treeview(self.manage_posts_tv, \
                                                                self.manage_posts_model, 2)
        self.change_status_combo.get_model().clear()
        if self.blogsystem == 'Wordpress':
            self.change_status_combo.insert_text(0, 'publish')                        
            self.change_status_combo.insert_text(1, 'draft')
            self.change_status_combo.insert_text(2, 'pending')
            self.change_status_combo.insert_text(3, 'private') 
        elif self.blogsystem == 'Blogger':
            self.change_status_combo.insert_text(0, 'publish')                        
            self.change_status_combo.insert_text(1, 'draft')
        
        self.change_status_combo.child.set_text(status)

        self.manage_posts_ntbk.set_current_page(1)

    def on_change_status_to_post(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Updating status')
        thread.start_new(self._do_change_status_to_post, (banner, widget))    
    
    def _do_change_status_to_post(self, banner, widget):
        from wordpy import posting
        try:
            #self.activate_led()
            postid, seliter, selmodel = utils.get_selected_from_treeview(self.manage_posts_tv, \
                                                                    self.manage_posts_model, 1) 
            
            status = self.change_status_combo.get_active_text()
            
            if self.blogsystem == 'Wordpress':
                posting.edit_post_status_wordpress(self.url, self.user, self.password, \
                                                    postid, status)
            elif self.blogsystem == 'Blogger':
                posting.edit_post_status_blogger(self.user, self.password, self.blogid, \
                                                    postid, status)
                                                            
            title = self.manage_posts_model.get_value(seliter, 0)
            if seliter:         
                self.manage_posts_model.set(seliter, 0, title, 1, postid, 2, status) 
                
            self.manage_posts_ntbk.set_current_page(0)
            banner.destroy()
            #self.stop_led()
        except Exception, e:       
            error_msg = 'Error updating the post status'
            self.show_threaded_error(banner, widget, error_msg, e)
            #self.stop_led()

    ## Manage Media Functions ##
    def show_img_description(self, widget):
        if not utils.is_treeview_selected(self.imagelist_tv):
            return
            
        # Setting the correct value to the widgets:
        self.set_description_entries()
                      
        if self.managing_img == 'flickr':
            self.media_ntbk.set_current_page(2)
        elif self.managing_img == 'picasa':
            self.media_ntbk.set_current_page(3) 

    def set_description_entries(self):
        selection = self.imagelist_tv.get_selection()

        if self.managing_img == 'flickr':     
            if selection.count_selected_rows() > 1: 
                #print 'multiple'
                self.batch_flickr_label.set_text('You are tagging a batch of images')
                self.img_title_entry.set_text('')
                self.img_tags_entry.set_text('')
                self.img_desc_buffer.set_text('')
                self.public_img_rb.set_active(True)
                self.do_private_img_widgets_insensitive()
                self.friend_img_rb.set_active(False)
                self.family_img_rb.set_active(False)
                for i in range(len(self.sets_model)):
                    self.sets_model[i][0] = False
                
                return         
            else:                        
                self.batch_flickr_label.set_text('')
                
                selmodel, selected = selection.get_selected_rows()
                iters = [selmodel.get_iter(path) for path in selected]    
                seliter = iters[0]
                filename = self.imglist_model.get_value(seliter, 2)    
        
                for i in range(len(all_images)):
                    if all_images[i][0] == filename:           
                        image = all_images[i]
                        break 

                self.img_title_entry.set_text(image[1])
                       
                self.img_tags_entry.set_text(image[3])
                self.img_desc_buffer.set_text(image[2])        
                   
                #image = [filename, title, desc, tags, public, friend, family, sets]
                
                if image[4] == 1:
                    self.public_img_rb.set_active(True)
                    self.do_private_img_widgets_insensitive()
                    self.friend_img_rb.set_active(False)
                    self.family_img_rb.set_active(False)
                else:
                    self.public_img_rb.set_active(False)
                    self.private_img_rb.set_active(True)
                    if image[5] == 1:
                        self.friend_img_rb.set_active(True)
                    else:
                        self.friend_img_rb.set_active(False)            
                    if image[6] == 1:
                        self.family_img_rb.set_active(True)
                    else:
                        self.family_img_rb.set_active(False)       

                try:
                    for j in range(len(self.sets_model)):
                        if self.sets_model[j][1] in image[7]:
                            self.sets_model[j][0] = True
                        else:
                            self.sets_model[j][0] = False
                except:
                    print 'Failed to get the sets.'

        elif self.managing_img == 'picasa':
            if selection.count_selected_rows() > 1: 
                #print 'multiple'
                self.batch_picasa_label.set_text('You are tagging a batch of images')
                self.picasa_title.set_text('')
                self.picasa_tags.set_text('')
                self.picasa_desc_buffer.set_text('')
                return         
            else:         
                self.batch_picasa_label.set_text('')
                
                selmodel, selected = selection.get_selected_rows()
                iters = [selmodel.get_iter(path) for path in selected]    
                seliter = iters[0]
                filename = self.imglist_model.get_value(seliter, 2)    
        
                for i in range(len(all_images)):
                    if all_images[i][0] == filename:           
                        image = all_images[i]
                        break        
                   
                self.picasa_title.set_text(image[1])
                self.picasa_tags.set_text(image[3])
                self.picasa_desc_buffer.set_text(image[2])
                       
                if image[4] == '':
                    df_channel = self.picasa_default_album_cb.get_active_text()
                else:
                    df_channel = image[4]
                    
                self.picasa_album_cb.child.set_text(df_channel)            

    def on_desc_flickr_ntbk_changed(self, widget, dummy, pagenum):
        if pagenum == 0:
            self.sets_btn.show()
            self.privacy_btn.show()
            self.description_btn.hide()
        elif pagenum == 1:
            self.sets_btn.hide()
            self.privacy_btn.show()
            self.description_btn.show()      
        elif pagenum == 2:
            self.sets_btn.show()
            self.privacy_btn.hide()
            self.description_btn.show()  

    def on_public_img_active(self, widget):
        if self.public_img_rb.get_active():
            self.do_private_img_widgets_insensitive()
        else:
            self.friend_img_rb.set_sensitive(True)
            self.family_img_rb.set_sensitive(True)

    def do_private_img_widgets_insensitive(self):
        self.friend_img_rb.set_sensitive(False)
        self.family_img_rb.set_sensitive(False)
        self.family_img_rb.set_active(False)
        self.friend_img_rb.set_active(False)
      
    def on_get_flickr_sets(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Getting Flickr sets')
        thread.start_new(self._do_get_flickr_sets, (banner, widget))      
     
    def _do_get_flickr_sets(self, banner, widget):
        from wordpy import flickr
        try:
            myflickr = flickr.Flickr('')
            setlist = myflickr.get_set_list()
            banner.destroy()
        except Exception, e:
            error_msg = 'Error getting Flickr sets'
            self.show_threaded_error(banner, widget, error_msg, e)
            setlist = []
            
        self.sets_model.clear()
        #setlist = [(id, name)]
        self.sets_model = self.__create_sets_model(setlist)
        self.sets_tv.set_model(self.sets_model)

    def __create_sets_model(self, setlist):
        lstore = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, \
                    gobject.TYPE_STRING)
        for item in setlist:
            iter = lstore.append()
            lstore.set(iter, 0, False, 1, item[0], 2, item[1])

        return lstore     

    def _sets_tv_columns(self, treeview):
        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.sets_toggled)
        column = gtk.TreeViewColumn('', renderer, active=0)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_fixed_width(50)
        treeview.append_column(column)

        column = gtk.TreeViewColumn('Id', gtk.CellRendererText(), text=1)
        column.set_visible(False)
        treeview.append_column(column)

        column = gtk.TreeViewColumn("Set", gtk.CellRendererText(), text=2)            
        treeview.append_column(column)

    def sets_toggled(self, cell, path):
        seliter = self.sets_model.get_iter((int(path),))
        fixed = self.sets_model.get_value(seliter, 0)

        fixed = not fixed
        self.sets_model.set(seliter, 0, fixed)   

    def on_upload_images(self, widget):
        #self.cancel_media_upload = False
        if self.managing_img == 'picasa':        
            for image in all_images:
                if image[1] == '':
                    utils.show_info_banner(widget, "Please, add a title a your images")
                    return
        
        self.upinfo_title.set_markup('<b>Uploading files...</b>')
        self.media_ntbk.set_current_page(4)
        self.timer = gobject.timeout_add (100, self.progress_timeout)
        thread.start_new(self.do_upload_media, ()) 
           
    def progress_timeout(self):
        self.upinfo_pgbar.pulse()
        return True
        
    def destroy_progress(self):
        if self.timer > 0:
            gobject.source_remove(self.timer)
        self.timer = 0   
        
    def do_upload_media(self):
        from wordpy import flickr, picasa
        #photos(flickr) = [[filename, title, desc, tags, public, friend, family, [sets_id]]]
        #photos(picasa) = [[filename, title, desc, tags, album]]
        #self.activate_led()
        try:
            filename_lb = self.upinfo_filename
            if self.managing_img == 'flickr':
                myflickr = flickr.Flickr('')
                myflickr.upload(all_images, filename_lb)
            elif self.managing_img == 'picasa':
                mypicasa = picasa.Picasa(self.picasa_login, self.picasa_pass)
                mypicasa.uploadPhotos(all_images, filename_lb)

            self.destroy_progress()
            self.on_upload_complete('Upload complete')
            #self.stop_led()
        except Exception, e:
            log = 'ERROR uploading media:\n%s' % e
            io.set_log_to_file(logfile, log)
            self.destroy_progress()
            self.on_upload_complete('Upload failed')
            #self.stop_led()

    def on_upload_complete(self, upinfo):
        self.upinfo_pgbar.hide()
        #self.cancel_upinfo_btn.hide()
        self.upinfo_title.set_markup('<b>'+upinfo+'</b>')
        self.upinfo_filename.set_text('')
        self.back_upinfo_btn.show()
        self.home_btn.set_sensitive(True)

    def on_media_ntbk_changed(self, widget, dummy, page):        
        if page == 6:       
            self.upinfo_pgbar.show()
            self.back_upinfo_btn.hide()
            self.home_btn.set_sensitive(False)

    def save_img_description(self, widget):
        selection = self.imagelist_tv.get_selection()
        selmodel, selected = selection.get_selected_rows()
        iters = [selmodel.get_iter(path) for path in selected] 
             
        if self.managing_img == 'flickr':
            sets = []

            for i in range(len(self.sets_model)):
                if self.sets_model[i][0] == True:        
                    #print self.sets_model[i][1]
                        sets.append(self.sets_model[i][1])
          
            title = self.img_title_entry.get_text()
            tags = self.img_tags_entry.get_text()
                
            start, end = self.img_desc_buffer.get_bounds()   
            desc = self.img_desc_buffer.get_text(start, end) 
                 
            if self.public_img_rb.get_active():
                public = 1
                family = 0
                friend = 0
            elif self.private_img_rb.get_active():
                public = 0
                if self.friend_img_rb.get_active():
                    friend = 1
                else:
                    friend = 0
                if self.family_img_rb.get_active():
                    family = 1
                else:
                    family = 0

            for seliter in iters:         
                filename = self.imglist_model.get_value(seliter, 2)
                image = [filename, title, desc, tags, public, friend, family, sets]
                for i in range(len(all_images)):
                    if all_images[i][0] == filename:           
                        all_images[i] = image
                        break  
            
        elif self.managing_img == 'picasa':    
            title = self.picasa_title.get_text()
            tags = self.picasa_tags.get_text()
            
            start, end = self.picasa_desc_buffer.get_bounds()   
            desc = self.picasa_desc_buffer.get_text(start, end) 

            album = self.picasa_album_cb.get_active_text()

            for seliter in iters:         
                filename = self.imglist_model.get_value(seliter, 2)
                image = [filename, title, desc, tags, album]
                for i in range(len(all_images)):
                    if all_images[i][0] == filename:           
                        all_images[i] = image
                        break 
        
        self.media_ntbk.grab_focus()
        self.media_ntbk.set_current_page(1)      
    
    def add_img_to_list(self, widget):
        imgfiles = utils.file_browser(self.window, gtk.FILE_CHOOSER_ACTION_OPEN, \
                            "Select File", self.path_add_img, "", "", True)                            
        if not imgfiles:
            return

        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Adding files, be patient please.')
        thread.start_new_thread(self._do_add_img_to_list, (imgfiles, banner))    
    
    def _do_add_img_to_list(self, imgfiles, banner): 
        self.path_add_img = os.path.dirname(imgfiles[len(imgfiles)-1])
        default_album = self.picasa_default_album_cb.get_active_text()
        file_error = False
        for filename in imgfiles:
            try:
                pixbuf = utils.file_to_pixbuf(filename, 'small')
                if filename is not None:
                    niter = self.imglist_model.append()            
                    self.imglist_model.set(niter, 0, pixbuf, 1, os.path.basename(filename), 2, filename)
                    del pixbuf
                    if self.managing_img == 'flickr':
                        image = [filename, '', '', '', 1, 0, 0, []]
                    elif self.managing_img == 'picasa':
                        image = [filename, '', '', '', default_album] 
                    all_images.append(image)
            except Exception, e:
                log = 'Error adding some files to the image list:\n%s' % e
                io.set_log_to_file(logfile, log)
                file_error = True

        banner.destroy()
        if file_error:
            utils.show_info_banner(self.window, "Some files weren't added")

    def delete_img_from_list(self, widget):
        if not utils.is_treeview_selected(self.imagelist_tv):
            return
            
        selection = self.imagelist_tv.get_selection()
        selmodel, selected = selection.get_selected_rows()
        iters = [selmodel.get_iter(path) for path in selected] 
        
        for i in iters:
            selname = selmodel.get_value(i, 2)
            selmodel.remove(i) 

            for i in range(len(all_images)):
                if all_images[i][0] == selname:          
                    del all_images[i]
                    break 

    def __create_imglist_model(self, imgfile):
        lstore = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING)
        #niter = lstore.append()
        #lstore.set(niter, 0, imgfile)
        return lstore
        
    def _imglist_tv_columns(self, treeview):       
        renderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn('', renderer, pixbuf=0)
        treeview.append_column(column)  
        
        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(), text=1)
        treeview.append_column(column)
        
        column = gtk.TreeViewColumn('Filename', gtk.CellRendererText(), text=2)
        column.set_visible(False)
        treeview.append_column(column)        

    ## Posting functions ##
           
    def progress_posting_timeout(self):
        self.posting_pgbar.pulse()
        return True
        
    def destroy_progress_posting(self):
        if self.timer_posting > 0:
            gobject.source_remove(self.timer_posting)
        self.timer_posting = 0   

    def on_posting_complete(self, upinfo):
        self.posting_title.set_markup('<b>'+upinfo+'</b>')
        self.posting_info.set_text('')
        self.posting_pgbar.hide()
        self.home_btn.set_sensitive(True)
        if upinfo == 'Error publishing':
            self.posting_save_btn.show()        
            self.posting_try_btn.show()
        else:
            self.recentposts.set_text('What else do you want to do?')
            self.main_ntbk.set_current_page(0)
            self.recentposts_title.set_markup('<b>Publish successful</b>')
            self.on_new_post(self.window, 'publish')

    def publish_post(self, widget):   
        if self.title_entry.get_text() == '':
            utils.show_info_banner(widget, 'You need write a title')
            return

        self.main_ntbk.set_current_page(14)
        self.posting_title.set_markup('<b>Publishing post...</b>')
        self.posting_info.set_label("Uploading content")       
        self.posting_pgbar.show()
        self.timer_posting = gobject.timeout_add (100, self.progress_posting_timeout)     
        thread.start_new(self._do_upload_post, (widget,))

    def _do_upload_post(self, widget):
        from wordpy import posting     
        try:
            #self.activate_led()   

            if self.is_edited:
                edited = True
                date = self.date
                postid = self.postid
            else:
                edited = False
                date = None
                postid = 0
           
            # Get recent tags chosen
            rtags = self.on_set_recenttags_to_post()
            #print 'rtags: ', rtags

            sch_time = self.hoursNE.get_value() + ':' + self.minutesNE.get_value()
            sch_date = self.date_schedule_entry.get_text()

            if self.blogsystem == 'Wordpress':                
                postid = posting.to_wordpress(self.url, self.user, self.password, self.title_entry, \
                            self.postpassword_entry, self.tags_entry, self.pings_cb, \
                            self.comments_cb, self.postBuffer, self.extendedBuffer, \
                            self.publishopt_combo, self.slug_entry, self.cats_model, \
                            self.tbacks_entry, self.techtags_entry, edited, postid, \
                            rtags, self.posting_info, sch_time, sch_date, self.schedule_cbtn)
            elif self.blogsystem == 'Blogger':
                postid = posting.to_blogger(self.url, self.user, self.password, self.title_entry, \
                            self.postBuffer, self.techtags_entry, self.tags_entry, \
                            self.publishopt_combo, self.blogid, edited, postid, rtags, \
                            self.posting_info, sch_time, sch_date, self.schedule_cbtn)
            elif self.blogsystem == 'Livejournal':
                postid = posting.to_livejournal(self.user, self.password, self.postBuffer, \
                                    self.title_entry, self.tags_entry, date, edited, \
                                    postid, rtags, self.comments_cb, sch_time, sch_date)

            if postid == None:
                self.destroy_progress_posting()
                self.on_posting_complete('Error publishing')                    
                log = 'ERROR publishing a post:\n%s' % 'Authentication failed'
                io.set_log_to_file(logfile, log)
                #self.stop_led()

            else: 
                self.is_edited = False
                self.post_edited_btn.hide()
                recentposts = RecentPosts()
                recentposts.put(self.title_entry.get_text(), self.accountname)
                
                recenttags = RecentTags()
                recenttags.put(self.tags_entry.get_text())
                
                #self.stop_led()

                self.destroy_progress_posting()
                self.on_posting_complete('Publish succesful')

        except Exception, e:
            log = 'ERROR publishing a post:\n%s' % e
            io.set_log_to_file(logfile, log)
            self.destroy_progress_posting()
            self.on_posting_complete('Error publishing') 
            #self.stop_led()                       

    def activate_led(self):
        if not scratchbox:
            color_file = open("/sys/devices/platform/i2c_omap.2/i2c-0/0-0032/color","w")
            color_file.write("29:f3:e1")
            color_file.close()
            mode_file = open("/sys/devices/platform/i2c_omap.2/i2c-0/0-0032/mode","w")
            mode_file.write('direct')
            mode_file.close()                

    def stop_led(self):
        if not scratchbox:    
            mode_file = open("/sys/devices/platform/i2c_omap.2/i2c-0/0-0032/mode","w")
            mode_file.write('run')
            mode_file.close()  

    def select_local_image(self, widget):
        filename = utils.file_browser(self.window, gtk.FILE_CHOOSER_ACTION_OPEN, \
                                        "Browse for Images", self.path_post_img, \
                                        '', '', False)
        if not filename:
            return

        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Opening image')
        thread.start_new_thread(self._do_select_local_image, (filename, widget, banner))

    def _do_select_local_image(self, filename, widget, banner):
        try:
            self.path_post_img = os.path.dirname(filename)
            self.local_img_label.set_text('file://' + filename)
            self.edit_pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
            self.set_html_tags_image_size()
            thumbnail_pixbuf = utils.file_to_pixbuf(filename, 'small')
            self.local_image_img.set_from_pixbuf(thumbnail_pixbuf)
            del thumbnail_pixbuf
            self.local_image_img.show()
            #print 'Image chosen: ', self.local_img_label.get_text()
            self.edit_img_btn.set_sensitive(True)
            banner.destroy()
        except Exception, e:
            error_msg = 'Error selecting a local image'
            self.show_threaded_error(banner, widget, error_msg, e)  

    def on_original_active(self, widget):
        ## If original in html size is active or not, do sensitive 
        ## or insesitive widgets
        if self.original_button.get_active():
            self.do_resize_widgets_insensitive()
        else:
            self.WidthNE.set_sensitive(True)
            self.labelwidth_html.set_sensitive(True)
            self.labelheight_html.set_sensitive(True)
            self.HeightNE.set_sensitive(True)
            self.ratio_btn.set_sensitive(True)	
				
    def do_resize_widgets_insensitive(self):
        ## Do insensitive the html size widgets
        self.WidthNE.set_sensitive(False)
        self.labelwidth_html.set_sensitive(False)
        self.labelheight_html.set_sensitive(False)
        self.HeightNE.set_sensitive(False)
        self.ratio_btn.set_sensitive(False)	

    def aspect_ratio_toggled(self, togglebutton, width, height, function):
        ## If there is local image selected, keep aspect ratio or not
        ## this is in html size widgets
        if function == 'resize':
            self.resize_preserve_aspect = togglebutton.get_active()
            preserve_aspect = self.resize_preserve_aspect
        else:
            self.preserve_aspect  = togglebutton.get_active()
            preserve_aspect = self.preserve_aspect
        try:
            if preserve_aspect:
                # Set height based on width and aspect ratio
                target_value = float(width.get_text())/self.local_image_img.get_pixbuf().get_width()
                target_value = int(target_value * self.local_image_img.get_pixbuf().get_height())
                self.ignore_preserve_aspect_callback = True
                height.set_text(str(target_value))
                self.ignore_preserve_aspect_callback = False
        except:
            print 'Image not selected.'
	
    def preserve_image_aspect(self, entry, option, entry2, function):
		## If there is local image selected, and keep aspect ratio active
		## in html size widgets, preserve image aspect
        try:
            if function == 'resize':
                if not self.resize_preserve_aspect:
                    return            
            else:
                if not self.preserve_aspect:
                    return
            if self.ignore_preserve_aspect_callback:
                return
            if option == "width":
                target_value = float(entry.get_text())/self.local_image_img.get_pixbuf().get_width()
                target_value = int(target_value * self.local_image_img.get_pixbuf().get_height())                     
            else:
                target_value = float(entry.get_text())/self.local_image_img.get_pixbuf().get_height()
                target_value = int(target_value * self.local_image_img.get_pixbuf().get_width())
                
            self.ignore_preserve_aspect_callback = True
            entry2.set_text(str(target_value))
            self.ignore_preserve_aspect_callback = False
        except:
            print 'Image not selected.'

    def on_insert_img_to_post(self, widget):
        imgurl = self.local_img_label.get_text()
        if imgurl == '':
            #If url is '' not pixbuf, not temp images should exist.
            self.delete_temp_images()
            #del self.local_image_img.get_pixbuf()
            self.textview_with_focus.grab_focus()
            self.return_to_textview_window()
            return

        add_img_buffer = self.textview_with_focus.get_buffer()
        #y si no es creado??
        #del self.local_image_img.get_pixbuf()
        
        if imgurl.find('/var/tmp/') >= 0:
        ## If there is temporal edited images, copy it to maemowordpy config dir
        ## and get it url, and delete the temporal image
        ## When it is uploaded with an article, it will be deleted
        ## it needs a the cached images function for deleting copied images that 
        ## haven't been upload with an article. For example, user edit an image, 
        ## and insert it in the article but she doesn't upload the article never, 
        ## the image will be in maemowordpy config dir forever ;)
            import shutil
            temp_image_dir = '/home/user/.maemowordpy/images/'
            shutil.copy(imgurl.replace('file://', ''), temp_image_dir)
            filename = imgurl.replace('file://', '').split('/')
            imgurl = 'file://' + temp_image_dir + filename[len(filename)-1]
            self.delete_temp_images()
            del self.edit_pixbuf
            self.edit_pixbuf = None

        img_src = '<img src="' + imgurl

        ## If resize html tags is active, add it to <img> tag
        if self.resize_button.get_active() == True:
            img_src = img_src + "\" width=\"" + self.WidthNE.get_value() + \
                                    "\" height=\"" + self.HeightNE.get_value()
        ## If there is alt text, add it to <img> tag		
        if self.en_alttext.get_text() != '':
            img_src = img_src + "\" alt=\"" + self.en_alttext.get_text()
        #final_img_src is the final <img src...> text
        final_img_src = img_src + '" />'

        if self.img_align.get_active_text() != '':
            text1 = '<div style="text-align:' + self.img_align.get_active_text()  + '">'
            add_img_buffer.insert_at_cursor(text1, -1)
            start, end = add_img_buffer.get_bounds()
            add_img_buffer.insert_interactive(end, final_img_src, -1)
            text2 = '</div>'
            start, end = add_img_buffer.get_bounds()			
            add_img_buffer.insert_interactive(end, text2, -1)        
        else:
            add_img_buffer.insert_at_cursor(final_img_src, -1)            
            
        self.textview_with_focus.grab_focus()
        self.return_to_textview_window()

    def get_flickr_photolist(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', 'Getting images')
        thread.start_new(self._do_get_flickr_photolist, (banner, widget))
    
    def _do_get_flickr_photolist(self, banner, widget):
        from wordpy import flickr
        self.flickr_photolist.clear()
        self.flickr_sizelist.clear()
        myflickr = flickr.Flickr('')
        page = int(self.flickrpageNE.get_value())
        try:
            self.photolist = myflickr.get_photolist(per_page=self.perpage, page=page)
            #list=photos, sizes, widths, heights, urls
            if self.photolist is not None:
                for item in self.photolist[0]:
                    self.flickr_photolist.append([item])
                self.combophotos.set_active(0)

                for item in self.photolist[1][0]:
                    self.flickr_sizelist.append([item])			
                self.combosize.set_active(0)

                self.labelWidth.set_text(str(self.photolist[2][0][0]))
                self.labelHeight.set_text(str(self.photolist[3][0][0])) 
                self.set_flickr_btn.set_sensitive(True)           
                banner.destroy()
            else:
                error_msg = 'There isnt so much images'        
                exc_msg = 'Error getting the Flickr images'
                self.show_threaded_error(banner, widget, error_msg, exc_msg)            
        except Exception, e:
            error_msg = 'Error getting the Flickr images'
            self.show_threaded_error(banner, widget, error_msg, e)  

    def on_sizelist_changed(self, widget):
        size_active = int(self.combosize.get_active())
        photo_active = int(self.combophotos.get_active())
        self.labelWidth.set_text(str(self.photolist[2][photo_active][size_active]))		 
        self.labelHeight.set_text(str(self.photolist[3][photo_active][size_active]))

    def on_photolist_changed(self, widget):
        photo_active = int(self.combophotos.get_active())
        size_active = int(self.combosize.get_active())
        self.labelWidth.set_text(str(self.photolist[2][photo_active][size_active])) 
        self.labelHeight.set_text(str(self.photolist[3][photo_active][size_active]))

    def on_set_flickr_photo(self, widget):
        photo = int(self.combophotos.get_active())
        size  = int(self.combosize.get_active())
        self.local_img_label.set_text(str(self.photolist[4][photo][size]))
        self.local_image_img.set_from_file(imgdir + 'img-missing.png')
        utils.show_info_banner(widget, "Image setted")

    def GetCacheImages(self):
        cached_images = []
        try:
            for image in os.listdir('/home/user/.maemowordpy/images/'):
                cached_images.append(image)
            return cached_images
        except Exception, e:
            log = 'ERROR getting cached images:\n%s' % e
            io.set_log_to_file(logfile, log)
            return None

    def cache_images_model(self, cached_images):
        lstore = gtk.ListStore(gobject.TYPE_STRING)
        for item in cached_images:
            iter = lstore.append()
            lstore.set(iter, 0, item)
        return lstore
        
    def __add_columns_cache_images(self, treeview): 
        renderer = gtk.CellRendererText()
        renderer.set_property('scale', 1.2)  
        column = gtk.TreeViewColumn('Cached Images', renderer, text=0)
        treeview.append_column(column)

    def view_cache_img(self, widget):
        try:
            if not utils.is_treeview_selected(self.cache_tv):
                return    
            selname, seliter, selmodel = utils.get_selected_from_treeview(self.cache_tv, \
                                                                self.cache_model, 0)            
            utils.preview_img('/home/user/.maemowordpy/images/' + selname) 
            
        except Exception, e:
            log = 'ERROR viewing cached images:\n%s' % e
            io.set_log_to_file(logfile, log)


    def delete_cache_img(self, widget):
        try:
            if not utils.is_treeview_selected(self.cache_tv):
                return    
            selname, seliter, selmodel = utils.get_selected_from_treeview(self.cache_tv, \
                                                                self.cache_model, 0)
            os.remove('/home/user/.maemowordpy/images/' + selname) 
            self.cache_model.remove(seliter)
        except Exception, e:
            log = 'ERROR deleting cached images:\n%s' % e
            io.set_log_to_file(logfile, log)

    def on_authotize_to_flickr(self, widget):
        from wordpy import flickr
        try:
            flickr.authorize_on_flickr(self.window)
        except Exception, e:
            log = 'ERROR authorizing on Flickr:\n%s' % e
            io.set_log_to_file(logfile, log) 

    def get_picasa_albumlist(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Getting album list')
        thread.start_new(self._do_get_picasa_albumlist, (banner, widget))      
    
    def _do_get_picasa_albumlist(self, banner, widget):
        from wordpy import picasa
        self.picasa_albumlist.clear()
        try:
            mypicasa = picasa.Picasa(self.picasa_login, self.picasa_pass)
            albums, albumlist = mypicasa.getAlbums()
            
            for item in albumlist:
                self.picasa_albumlist.append([item])
                
            self.picasa_default_album_cb.set_active(0)
            self.picasa_albums_cb.set_active(0)            
            banner.destroy()
        except Exception, e:
            error_msg = 'Error getting Picasa album list'
            self.show_threaded_error(banner, widget, error_msg, e)

    def get_picasa_photolist(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Getting Picasa images')
        thread.start_new(self._do_get_picasa_photolist, (banner, widget))     
    
    def _do_get_picasa_photolist(self, banner, widget):
        from wordpy import picasa
        self.picasa_photolist.clear() 
        album_title = self.picasa_albums_cb.get_active_text()     
        try:           
            mypicasa = picasa.Picasa(self.picasa_login, self.picasa_pass)
            self.picasa_photos = mypicasa.getPhotosInAlbum(album_title)
            
            if self.picasa_photos is not None:
                for item in self.picasa_photos:
                    self.picasa_photolist.append([item[0]])
                
                if self.picasa_photos != []:
                    self.picasa_photos_cb.set_active(0)   
                    self.picasa_width.set_text(self.picasa_photos[0][2])
                    self.picasa_height.set_text(self.picasa_photos[0][3])  
                    self.set_picasa_btn.set_sensitive(True)
                    banner.destroy()
                else:
                    self.picasa_photos_cb.child.set_text('')   
                    self.picasa_width.set_text('')
                    self.picasa_height.set_text('') 
                    self.set_picasa_btn.set_sensitive(False)            
                    error_msg = 'No images in this album'
                    exc_msg = ''
                    self.show_threaded_error(banner, widget, error_msg, exc_msg)
            else:
                error_msg = 'Error getting Picasa photo list'
                exc_msg = 'Album not found'
                self.show_threaded_error(banner, widget, error_msg, exc_msg)                 
        except Exception, e:
            error_msg = 'Error getting Picasa photo list'
            self.show_threaded_error(banner, widget, error_msg, e)
       
    def on_picasaphotos_changed(self, widget):
        if self.picasa_photos != []:
            photo_active = int(self.picasa_photos_cb.get_active())
            self.picasa_width.set_text(str(self.picasa_photos[photo_active][2])) 
            self.picasa_height.set_text(str(self.picasa_photos[photo_active][3]))    
    
    def on_set_picasa_photo(self, widget):
        photo = int(self.picasa_photos_cb.get_active())
        self.local_img_label.set_text(str(self.picasa_photos[photo][1]))
        self.local_image_img.set_from_file(imgdir + 'img-missing.png')
        utils.show_info_banner(widget, "Image setted")

    ## Edit Local Images UI ##
    
    def on_resize_edit_img(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Resizing image')
        thread.start_new(self._do_resize_img, (widget, banner)) 

    def _do_resize_img(self, widget, banner):
        from wordpy import imaged
        if self.edit_pixbuf == None:
            self.show_threaded_error(banner, widget, 'Not image selected', '') 
            return
            
        width = int(self.ResizeWidthNE.get_value())
        height = int(self.ResizeHeightNE.get_value())
        self.edit_pixbuf = imaged.resize_image(widget, self.edit_pixbuf, width, height)
        
        self.edit_img_ntbk.set_current_page(0)
        banner.destroy()
        
    def set_html_tags_image_size(self):
        pixbuf = gtk.gdk.pixbuf_new_from_file(self.local_img_label.get_text().replace('file://', ''))
        
        #self.WidthNE.entry.set_text(str(self.edit_pixbuf.get_width()))
        self.WidthNE.entry.set_text(str(pixbuf.get_width()))
        self.ignore_preserve_aspect_callback = True
        #self.HeightNE.entry.set_text(str(self.edit_pixbuf.get_height()))
        self.HeightNE.entry.set_text(str(pixbuf.get_height()))
        self.ignore_preserve_aspect_callback = False
        del pixbuf
            
    def on_crop_img(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Cropping image')
        thread.start_new(self._do_crop_img, (widget, banner)) 

    def _do_crop_img(self, widget, banner):
        from wordpy import imaged
        if self.edit_pixbuf == None:
            show_threaded_error(self, banner, widget, 'Not image selected', '') 
            return        
        
        if self.rect != None:
            temp_pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, self.edit_pixbuf.get_has_alpha(), 8, self.coords[2], self.coords[3])
            self.edit_pixbuf.copy_area(self.coords[0], self.coords[1], self.coords[2], self.coords[3], temp_pixbuf, 0, 0)
            self.edit_pixbuf = temp_pixbuf
            del temp_pixbuf

            imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'edit', self.edit_img)

        self.add_img_ntbk.set_current_page(6)
        banner.destroy()

    def on_rotate_img(self, widget, orientation):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Rotating image')
        thread.start_new(self._do_rotate_img, (widget, banner, orientation)) 

    def _do_rotate_img(self, widget, banner, orientation):
        from wordpy import imaged
        if self.edit_pixbuf == None:
            self.show_threaded_error(banner, widget, 'Not image selected', '')           
            return

        if orientation == 'left':
            angle = 90
        elif orientation == 'right':
            angle = 270
        elif orientation == 'angle':
            angle = int(self.AngleNE.get_value())
        
        self.edit_pixbuf = imaged.rotate_image(widget, self.edit_pixbuf, angle)

        imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'edit', self.edit_img)

        banner.destroy()

    def on_flip_img(self, widget, mode):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                    'Flipping image')
        thread.start_new(self._do_flip_img, (widget, banner, mode)) 
            
    def _do_flip_img(self, widget, banner, mode):
        from wordpy import imaged
        if self.edit_pixbuf == None:
            self.show_threaded_error(banner, widget, 'Not image selected', '')
            return        
 
        self.edit_pixbuf = imaged.flip_image(widget, self.edit_pixbuf, mode)
        imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'edit', self.edit_img)
        banner.destroy()

    def get_screenshot(self, widget):
        self.screenshot_delay = int(self.SnapTimeNE.get_value())
        while gtk.events_pending():
            gtk.main_iteration()

        a = gobject.timeout_add(self.screenshot_delay*1000, self._do_screenshot)

        gobject.timeout_add(self.screenshot_delay*1000, self.edit_img_ntbk.set_current_page, 0)

    def _do_screenshot(self):
        from wordpy import imaged    
        self.edit_pixbuf = imaged.do_screenshot()
        imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'edit', self.edit_img)        

    def on_set_edited_img(self, widget):
        # Save as /tmp/maemowordpy-<random>/filename.png
        try:
            from wordpy import imaged
            tmpfile = imaged.create_temp_image(self.edit_pixbuf, widget)

            if tmpfile is None:
                return
        			
            self.local_img_label.set_text("file://" + str(tmpfile))
            self.set_html_tags_image_size()

            imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'small', self.local_image_img)
            self.add_img_ntbk.set_current_page(1)
            utils.show_info_banner(widget, "Image setted")
        except Exception, e:
            utils.show_info_banner(widget, "Error setting edited image")
            log = 'ERROR setting edited image:\n%s' % e
            io.set_log_to_file(logfile, log)

    def delete_temp_images(self):
        from wordpy import imaged
        imaged.delete_temp_images(self.window)

    def on_set_inet_img(self, widget):
        self.local_img_label.set_text(self.inet_img_entry.get_text())
        self.local_image_img.set_from_file(imgdir + 'img-missing.png')
        utils.show_info_banner(widget, "Image setted")

    def on_color_or_bright(self, widget):
        banner = hildon.hildon_banner_show_animation(widget, 'qgn_indi_pball_a', \
                                                        'Applying filter')
        thread.start_new_thread(self._do_color_or_bright, (widget, banner))
    
    
    def _do_color_or_bright(self, widget, banner):
        try:
            from wordpy import imaged        
            value = self.colorizeBar.get_value()/5.0

            if self.colorize_btn.get_property("visible"):   
                option = 'color'
            else:
                option = 'brightness'
            
            self.edit_pixbuf = imaged.do_colorize_or_brightness(widget, \
                                            self.edit_pixbuf, value, option)
            imaged.set_pixbuf_to_gtkimage(self.edit_pixbuf, 'edit', self.edit_img)
            banner.destroy()
        except Exception, e:
            error_msg = 'Error applying filter to image'
            self.show_threaded_error(banner, widget, error_msg, e)        

    def show_threaded_error(self, banner, widget, error_msg, exc_msg):
        banner.destroy()
        gtk.gdk.threads_enter()            
        utils.show_info_banner(widget, error_msg)
        gtk.gdk.threads_leave()   
        log = '%s:\n%s' % (error_msg, exc_msg)
        io.set_log_to_file(logfile, log)
        return  

class RecentPosts:
    def get(self):
        if os.path.exists(configdir + 'recentposts'):
            f = open(configdir + 'recentposts', 'r')
        else:
            f = open(configdir + 'recentposts', 'w') 
            f.close()
            f = open(configdir + 'recentposts', 'r')                       
        recentposts = f.read()

        if len(recentposts) == 0:
            recentposts = '(No recent posts)'
        f.close()
        
        return recentposts

    def put(self, title, blog):
        new = '<small>%s <b>(%s)</b></small>\n' % (title, blog)
        if os.path.exists(configdir + 'recentposts'):
            f = open(configdir + 'recentposts', 'r')
        else:
            f = open(configdir + 'recentposts', 'w')   
            f.close()
            f = open(configdir + 'recentposts', 'r') 
                           
        lines = f.readlines()
        f.close()

        if len(lines) == 3:
            lines[2] = lines[1]
            lines[1] = lines[0]
            lines[0] = new
        else:
            lines.insert(0, new)

        lines[len(lines)-1] = lines[len(lines)-1].replace('\n', '')
            
        f = open(configdir + 'recentposts', 'w')
        f.writelines(lines)
        f.close()                


class RecentTags:
    def get(self):
        if os.path.exists(configdir + 'recenttags'):
            f = open(configdir + 'recenttags', 'r')
        else:
            f = open(configdir + 'recenttags', 'w')
            f.close()
            f = open(configdir + 'recenttags', 'r')    
                                  
        recenttags = f.readlines()
        if len(recenttags) == 0:
             recenttags = []
        f.close()
        
        for i in range(len(recenttags)):
            recenttags[i] = recenttags[i].replace('\n', '')
        
        return recenttags

    def put(self, tag_txt):
        if os.path.exists(configdir + 'recenttags'):
            f = open(configdir + 'recenttags', 'r')
        else:
            f = open(configdir + 'recenttags', 'w')   
            f.close()
            f = open(configdir + 'recenttags', 'r') 

        recenttags = f.readlines()
        f.close()
        
        tags = []
        for i in tag_txt.split(", "):
            for j in i.split(" ,"):
                for k in j.split(","):
                    for l in k.split("; "):
                        for m in l.split(" ;"):
                            for n in m.split(";"):
                                if (n != " " and n != ""):
                                    tags.append(n)

        for tag in tags:
            if tag+'\n' not in recenttags:        
                if len(recenttags) == 20:
                    for i in range(len(recenttags)):
                        recenttags[len(recenttags)-1-i] = recenttags[len(recenttags)-2-i]
                        recenttags[0] = tag + '\n'
                else:
                    recenttags.insert(0, tag+'\n')
        
        f = open(configdir + 'recenttags', 'w')
        f.writelines(recenttags)
        f.close()         
 
if __name__ == "__main__":
    WordpyUI = WordpyUI()

    import getopt                              
    opts, args = getopt.getopt(sys.argv[1:], "t:p:", ["title=", "post="])
    for opt, arg in opts:
        if opt in ("-t", "--title"):
            WordpyUI.title_entry.set_text(arg)
        elif opt in ("-p", "--post"):
           WordpyUI. postBuffer.set_text(arg)
        if opt in ("-t", "--title") or opt in ("-p", "--post"):
            WordpyUI.main_ntbk.set_current_page(1)
            WordpyUI.window.present()

    loop = gobject.MainLoop()
    loop.run()    


