# Copyright (C) 2008 by Daniel Martin Yerga
# <dyerga@gmail.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from wordpy import io, utils
import gtk
import os

#FIXME if it's ran from scratchbox
logfile = '/home/user/.maemowordpy/log'

def resize_image(widget, pixbuf, width, height):
    try:
        pixbuf = pixbuf.scale_simple(int(width), int(height), gtk.gdk.INTERP_BILINEAR)
        return pixbuf
    except Exception, e:
        error_msg = 'Error resizing image'
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error resizing the image')        
        return pixbuf

def rotate_image(widget, pixbuf, angle):
    try:
        im = pixbuf2Image(pixbuf)
        rotated_im = im.rotate(angle)
        pixbuf = image2pixbuf(rotated_im)
        del im
        del rotated_im
        
        return pixbuf
    except Exception, e:
        error_msg = 'Error rotating image'
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error rotating the image')        
        return pixbuf

def flip_image(widget, pixbuf, mode):
    try:
        import Image   
        im = pixbuf2Image(pixbuf)
        if mode == 'horizontal':
            flip = Image.FLIP_LEFT_RIGHT
        elif mode == 'vertical':
            flip = Image.FLIP_TOP_BOTTOM
        
        flipped_im = im.transpose(flip)
        pixbuf = image2pixbuf(flipped_im)
        del im
        del flipped_im
        
        return pixbuf
    except Exception, e:
        error_msg = 'Error flipping image'
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error flipping the image')        
        return pixbuf

def invert_or_contour_image(widget, pixbuf, mode):
    try:
        if mode == 'invert':
            import ImageChops
            im = pixbuf2Image(pixbuf)
            new_im = ImageChops.invert(im)
        elif mode == 'contour':
            import ImageFilter
            im = pixbuf2Image(pixbuf)
            new_im = im.filter(ImageFilter.CONTOUR)

        pixbuf = image2pixbuf(new_im)
        del im
        del new_im
        
        return pixbuf
    except Exception, e:
        error_msg = 'Error doing %s to the image' % mode
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error doing %s to the image' % mode)        
        return pixbuf

def do_screenshot():
    try:
        from wordpy import imaged
        root_win = gtk.gdk.get_default_root_window()
        x = 0
        y = 0
        width = gtk.gdk.screen_width()
        height = gtk.gdk.screen_height()
        pix = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, width, height)
        pix = pix.get_from_drawable(root_win, gtk.gdk.colormap_get_system(), \
                                                    x, y, 0, 0, width, height)

        return pix
    except Exception, e:
        error_msg = 'Error making the screenshot'
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error making the screenshot')  


def do_colorize_or_brightness(widget, pixbuf, value, option):
    try:
        im = pixbuf2Image(pixbuf)
        import ImageEnhance
        if option == 'color':
            enh = ImageEnhance.Color(im)
            new_im = enh.enhance(value)
        elif option == 'brightness':
            enh = ImageEnhance.Brightness(im)
            new_im = enh.enhance(value)

        pixbuf = image2pixbuf(new_im)
        del new_im
        del enh
        del im

        return pixbuf
    except Exception, e:
        error_msg = 'Error doing %s to the image' % option
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error doing %s to the image' % option)        
        return pixbuf


def image2pixbuf(im):
    ## Convert PIL Image to gtk.Pixbuf
    import StringIO
    file1 = StringIO.StringIO()  
    im.save(file1, "ppm")  
    contents = file1.getvalue()  
    file1.close()
    loader = gtk.gdk.PixbufLoader("pnm")
    loader.write(contents, len(contents))  
    pixbuf = loader.get_pixbuf()  
    loader.close()  
    return pixbuf  

def pixbuf2Image(pixbuf):
    ## Convert gtk.Pixbuf to PIL Image
    import Image
    assert(pixbuf.get_colorspace() == gtk.gdk.COLORSPACE_RGB)
    dimensions = pixbuf.get_width(), pixbuf.get_height()
    stride = pixbuf.get_rowstride()
    pixels = pixbuf.get_pixels()
    mode = pixbuf.get_has_alpha() and "RGBA" or "RGB"

    return Image.frombuffer(mode, dimensions, pixels, "raw", mode, stride, 1)
        
def set_pixbuf_to_gtkimage(pixbuf, mode, gtkimage):
    w, h = utils.set_thumbnail_size(pixbuf, mode)
    scale_pix = pixbuf.scale_simple(int(w), int(h), gtk.gdk.INTERP_BILINEAR)
    gtkimage.set_from_pixbuf(scale_pix)
    gtkimage.show()
    del scale_pix

def create_temp_image(pixbuf, widget):
    # Save as /tmp/maemowordpy-<random>/random1.jpg
    import random
    import string
    import tempfile
    try:    
        random_name = ''.join([random.choice(string.letters) for x in xrange(8)])
        tmpdir = tempfile.mkdtemp(prefix='maemowordpy') + "/"
        tmpfile = tmpdir + random_name + '.jpg'
        pixbuf.save(tmpfile, "jpeg")
        return tmpfile
    except Exception, e:
        error_msg = 'Error creating temp image'
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error creating temp image')     
        return None


def delete_temp_images(widget):
    ## Delete temporal edited images ##
    try:     
        for i in os.listdir("/var/tmp"):
            if i.find("maemowordpy") >= 0:
                import shutil
                print 'Deleting temp directory: ', i
                shutil.rmtree("/var/tmp/" + i)
    except Exception, e:
        error_msg = 'Error deleting temp images'
        log = '%s:\n%s' % (error_msg, e)
        io.set_log_to_file(logfile, log)
        utils.show_info_banner(widget, 'Error deleting temp images')     
        return None
