# Copyright (C) 2008 by Daniel Martin Yerga
# <dyerga@gmail.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


import gtk, hildon
import os
import gobject

#FIXME if it's ran from scratchbox
logfile = '/home/user/.maemowordpy/log'

def is_treeview_selected(treeview):
    selection = treeview.get_selection()
    if selection.count_selected_rows() == 0:
        show_info_banner(treeview, 'No selected item') 
        return False
    else:
        return True

def get_selected_from_treeview(treeview, model, setting):
    selection = treeview.get_selection()
    selected_model, selected_iter = selection.get_selected()
    if selected_iter:
        selected_name = model.get_value(selected_iter, setting)
    return selected_name, selected_iter, selected_model
    
def show_confirmation(window, msg):
    dialog = gtk.MessageDialog(parent=window, type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO, message_format=msg)
    dialog.show_all()
    result = dialog.run()
    if result == -8:
        dialog.destroy()
        return True

    dialog.destroy()
    return False

def show_info_dialog(window, msg):
    dialog = hildon.hildon_note_new_information(window, msg)
    dialog.show_all()
    dialog.run()
    dialog.destroy()
      
def show_info_banner(widget, msg):
    hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', msg)       

def show_log_dialog(log):
    #Functions for the log
    def close_dialog(widget, dialog):
        dialog.destroy()
    def clear(widget, textview):
        textview.get_buffer().set_text('')
        f = open(logfile, 'w')
        f.close()
    def save(widget, textview):
        import cPickle
        try:
            start, end = textview.get_buffer().get_bounds() 
            logtext = textview.get_buffer().get_text(start, end)    
            save_file = open('/home/user/MyDocs/.documents/wordpy-log.txt', 'wb')
            cPickle.dump(logtext, save_file)
            save_file.close()
            show_info_banner(widget, 'Log file saved in Documents folder')
        except:
            show_info_banner(widget, 'Error saving the log file')
    #def send(widget, textview):
    #    start, end = textview.get_buffer().get_bounds() 
    #    logtext = textview.get_buffer().get_text(start, end)
    #    import os
    #    os.command('dbus-send --print-reply --type=method_call --dest=com.nokia.modest /com/nokia/modest com.nokia.modest.MailTo string:mailto:dyerga@gmail.com?body='+logtext)
        
    #Log dialog UI
    dialog = gtk.Dialog(title='Log', parent=None, flags=gtk.WIN_POS_CENTER)

    dialog.set_size_request(600, 350)
    swin = gtk.ScrolledWindow()
    swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    swin.set_shadow_type(gtk.SHADOW_IN)
    textview = gtk.TextView()
    textview.set_property("editable", False)
    textview.set_property("wrap-mode", gtk.WRAP_WORD)
    textview.set_property("cursor-visible", False)
    textview.get_buffer().set_text(log)
    swin.add(textview)

    dialog.vbox.pack_start(swin, True, True, 0)
    
    hbox = gtk.HBox()
    hbox.set_size_request(-1, 55)
    
    save_btn = gtk.Button('Save')
    save_btn.set_property("can-focus", False)
    save_btn.connect('clicked', save, textview)
    clear_btn = gtk.Button('Clear')
    clear_btn.set_property("can-focus", False)
    clear_btn.connect('clicked', clear, textview)
    #send_btn = gtk.Button('Send')
    #send_btn.set_property("can-focus", False)
    #send_btn.connect('clicked', send, textview)    
    close_btn = gtk.Button('Close')
    close_btn.set_property("can-focus", False) 
    close_btn.connect('clicked', close_dialog, dialog)
    
    hbox.pack_start(save_btn, True, True, 5)
    hbox.pack_start(clear_btn, True, True, 5)
    #hbox.pack_start(send_btn, True, True, 5)
    hbox.pack_start(close_btn, True, True, 5) 
           
    dialog.vbox.pack_start(hbox, False, False, 0) 
      
    dialog.show_all()
    dialog.run()
    dialog.destroy()


def file_browser(window, action, title, folder, filename, file_ext, multiple):
    file_dialog = hildon.FileChooserDialog(window, action=action)
    file_dialog.set_title(title)
    if multiple:
        file_dialog.set_select_multiple(True)
    file_dialog.set_default_response(gtk.RESPONSE_CANCEL)
    file_dialog.set_current_folder(folder)
    if (action==gtk.FILE_CHOOSER_ACTION_SAVE):
        file_dialog.set_current_name(filename)

    result = file_dialog.run()
    if result == gtk.RESPONSE_OK:
        if multiple:
            namefile = file_dialog.get_filenames()
        else:
            namefile = file_dialog.get_filename()            
        if (action==gtk.FILE_CHOOSER_ACTION_SAVE):
            namefile, extension = os.path.splitext(namefile)
            namefile = namefile + "." + file_ext
    else:
        namefile = False
    file_dialog.destroy()

    return namefile

def file_to_pixbuf(filename, mode):
    import gtk.gdk
    new_pixbuf = gtk.gdk.pixbuf_new_from_file(filename)

    ## Scale pixbuf
    w, h = set_thumbnail_size(new_pixbuf, mode)
    image_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, int(w), int(h))

    del new_pixbuf    
    
    return image_pixbuf
    
def set_thumbnail_size(pixbuf, mode):
    pix_width = pixbuf.get_width()
    pix_height = pixbuf.get_height()
		
    ##Calculate thumbnail size for vertical and horizontal images
    ##large_img for preview image dialog
    ##else: preview mini
    if mode == 'large':
        if pix_width > pix_height:
            w = 280
            h = float(pix_height)/float(pix_width) * 280
        elif pix_height > pix_width:
            h = 280
            w = float(pix_width)/float(pix_height) * 280
        elif pix_height == pix_width:
            h = 280
            w = 280
        else:
            print "Pixbuf size can't be calculated."		
				
    elif mode == 'small':
        if pix_width > pix_height:
            w = 140
            h = float(pix_height)/float(pix_width) * 140
        elif pix_height > pix_width:
            h = 140
            w = float(pix_width)/float(pix_height) * 140
        elif pix_height == pix_width:
            h = 140
            w = 140
        else:
            print "Pixbuf size can't be calculated."

    elif mode == 'edit':
        if pix_width > pix_height:
            w = 200
            h = float(pix_height)/float(pix_width) * 200
        elif pix_height > pix_width:
            h = 200
            w = float(pix_width)/float(pix_height) * 200
        elif pix_height == pix_width:
            h = 200
            w = 200
        else:
            print "Pixbuf size can't be calculated."    
    

    return w, h

def get_selection_iters(txtview):
    """This function gets the start and end selection
    iters from the text view.  If there is no selection
    the current position of the cursor will be returned.
    Returns - start,end - gtk.TextIter objects"""

    start = None
    end = None

    txtview.grab_focus()
    txtBuffer = txtview.get_buffer()

    #First check to see that the text buffer is valid
    if (not txtBuffer):
        show_info_banner(txtview, "Text buffer not available")
        return start,end

    #Get the selection bounds
    bounds = txtBuffer.get_selection_bounds();
    if (bounds):
        #If there is a selection we are done
        start,end = bounds
    else:
        #There is no selection so just get the cursor mark
        cursor_mark = txtBuffer.get_insert()
        """Set start and end to be gtk.TextIter objercts at the
        position of the cursor mark"""
        start = txtBuffer.get_iter_at_mark(cursor_mark)
        end = txtBuffer.get_iter_at_mark(cursor_mark)

    return start, end

def insert_text(txtview, text):
    """This function inserts text into the text buffer
    self.txtBuffer at the current selection.  If text is
    selected it will be overwritten, otherwise it will simply be
    inserted at the cursor position
    text - The text to be inserted in the buffer
    """

    txtview.grab_focus()
    txtBuffer = txtview.get_buffer()

    start, end = get_selection_iters(txtview);

    if ((not start)or(not end)):
        show_info_banner(txtview, "Error inserting text")
        return;

    #Delete the selected text (start and end will be equal after)
    txtBuffer.delete(start,end)
    #Save a mark at the start position since after we insert
    #the text start will be invalid
    start_mark = txtBuffer.create_mark(None, start, True)
    #Insert, end will be set to the end insert position
    txtBuffer.insert(end,text)
    start = txtBuffer.get_iter_at_mark(start_mark)
    #select the text, use end as the first param so that
    #it will be the cursor position
    txtBuffer.select_range(end,start)
    #delete the start mark
    txtBuffer.delete_mark(start_mark)

def wrap_selection(txtview, start_tag, end_tag):
    """This fucntion is used to wrap the currently selected
    text in the gtk.TextView with start_tag and end_tag. If
    there is no selection start_tag and end_tag will be
    inserted at the cursor position
    start_tag - The text that will go at the start of the
    selection.
    end_tag - The text that will go at the end of the
    selection."""

    txtview.grab_focus()
    txtBuffer = txtview.get_buffer()

    start, end = get_selection_iters(txtview);
    if ((not start)or(not end)):
        show_info_banner(txtview, "Error inserting text")
        return;
        
    #Create a mark at the start and end
    start_mark = txtBuffer.create_mark(None,start, True)
    end_mark = txtBuffer.create_mark(None, end, False)
    #Insert the start_tag
    txtBuffer.insert(start, start_tag)
    #Get the end iter again
    end = txtBuffer.get_iter_at_mark(end_mark)
    #Insert the end tag
    txtBuffer.insert(end, end_tag)
    #Get the start and end iters
    start = txtBuffer.get_iter_at_mark(start_mark)
    end = txtBuffer.get_iter_at_mark(end_mark)
    #Select the text
    txtBuffer.select_range(end,start)
    #Delete the gtk.TextMark objects
    txtBuffer.delete_mark(start_mark)
    txtBuffer.delete_mark(end_mark)
    
def preview_img(filename):
    view_image_dlg = gtk.Dialog()
    
    view_image_dlg.set_title('View cached image')
    view_image_dlg.add_button("Close", gtk.RESPONSE_CLOSE)

    view_img = gtk.Image()
    view_image_dlg.vbox.pack_start(view_img)
    
    scale_pix = file_to_pixbuf(filename, 'large')

    view_img.set_from_pixbuf(scale_pix)
    view_image_dlg.show_all()

    del scale_pix

    view_image_dlg.run()
    view_image_dlg.destroy()

#This is a finger friendly number editor
#It's a simple gtk.Entry with two gtk.Button
class FingerNE(gtk.HBox):
    def __init__ (self, orientation):
        gtk.HBox.__init__ (self)

        self.step = 1
        self.minimum = False
        self.maximum = False
        self.entry = gtk.Entry()
        self.entry.grab_focus()
        self.entry.set_property('editable', False)
        self.entry.set_property('xalign', 0.5)
        self.entry.set_text(str(0))

        self.plus = gtk.Button()
        self.plus.set_property('focus-on-click', False)
        self.plus.connect('clicked', self.plus_clicked)
        self.plus.connect('pressed', self.press_button, 'more')
        self.plus.connect('released', self.release_button)

        self.minus = gtk.Button()
        self.minus.set_property('focus-on-click', False)        
        self.minus.connect('clicked', self.minus_clicked)
        self.minus.connect('pressed', self.press_button, 'minus')
        self.minus.connect('released', self.release_button)
        
        self.mybox = self.set_orientation(orientation)
        
        self.add(self.mybox)

    #Set the image for the buttons, generally + and -
    def set_image_buttons(self, plus_imgfile, minus_imgfile):
        minus_img = gtk.Image()
        minus_img.set_from_file(minus_imgfile)     
        self.minus.set_image(minus_img)
        
        plus_img = gtk.Image()
        plus_img.set_from_file(plus_imgfile)
        self.plus.set_image(plus_img)        

        minus_img.show()
        plus_img.show()
        self.minus.show()
        self.plus.show()
        
    def do_realize (self):
        gtk.HBox.do_realize (self)

    def press_button(self, widget, option):
        self.start_spinning(widget, option)

    def release_button(self, widget):
        self.stop_spinning()

    def stop_spinning(self):
        if (self.timer > 0):
            gobject.source_remove(self.timer)
        self.timer = 0
        self.need_timer = False

    def start_spinning(self, widget, option):
        self.timer_step = self.step
        self.need_timer = True
        self.timer = gobject.timeout_add (50, self.spin_timer, widget, option)
 
    def spin_timer(self, widget, option):
        is_min = False
        is_max = False
        if self.timer:
            if option == 'more':
                new_value = int(self.get_value()) + self.step
                is_max = self.is_maximum(widget, new_value)
            elif option == 'minus':
                new_value = int(self.get_value()) - self.step             
                is_min = self.is_minimum(widget, new_value)

            if is_min or is_max:
                self.stop_spinning()
            else:
                self.timer = gobject.timeout_add (50, self.spin_timer, widget, option)

        if self.need_timer:
            self.need_timer = False
            self.timer = gobject.timeout_add (50, self.spin_timer, widget, option)

    #When the + button is clicked
    def plus_clicked(self, widget):
        self.is_maximum(widget, self.get_value())
 
    #When the - button is clicked
    def minus_clicked(self, widget):    
        self.is_minimum(widget, self.get_value())            

    def is_maximum(self, widget, new_value):
        if self.maximum is not False:
            if int(new_value) >= self.maximum:
                hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', \
                                                    'Maximum value reached')
                return True
            else:
                try:            
                    self.set_value(int(new_value))
                    return False
                except ValueError:
                    self.set_value(0)
                    hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', \
                                                        'Invalid literal value') 
                    return True
        else:
            try:        
                self.set_value(int(new_value))
                return False
            except ValueError:
                self.set_value(0)
                hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', \
                                                    'Invalid literal value')
                return True

    def is_minimum(self, widget, new_value):
        if self.minimum is not False:
            if int(new_value) <= self.minimum:
                hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', \
                                                        'Minimum value reached')
                return True
            else:
                try:
                    self.set_value(int(new_value))
                    return False
                    
                except ValueError:
                    self.set_value(self.minimum)
                    hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', \
                                                        'Invalid literal value')
                    return True
        else:
            try:
                self.set_value(int(new_value))
                return False
                
            except ValueError:
                self.set_value(self.minimum)
                hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', \
                                                    'Invalid literal value')     
                return True
                 
    #Maximum value for the widget
    def set_maximum(self, maximum):
        self.maximum = maximum + 1

    #Minimum value for the widget    
    def set_minimum(self, minimum):
        self.minimum = minimum - 1

    #Step for the widget           
    def set_step(self, step):
        self.step = step

    #Set the size for the entry and the buttons
    def set_size(self, size):
        self.entry.set_size_request(size[0], size[1])
        self.plus.set_size_request(size[2], size[3])
        self.minus.set_size_request(size[2], size[3])

    #Set the value for the entry
    def set_value(self, value):
        self.entry.set_text(str(value))

    #Return the value for the entry
    def get_value(self):
        value = self.entry.get_text()
        return value

    #Orientation for the widget, vertical or horizontal
    def set_orientation(self, orientation):
        if orientation == 'horizontal':
            mybox = gtk.HBox()
            mybox.pack_start(self.minus, False, False, 0)
            mybox.pack_start(self.entry, False, False, 0)  
            mybox.pack_start(self.plus, False, False, 0)              
            
        elif orientation == 'vertical':
            mybox = gtk.VBox()
            mybox.pack_start(self.plus, False, False, 0)
            mybox.pack_start(self.entry, False, False, 0) 
            mybox.pack_start(self.minus, False, False, 0)             

        return mybox

    def show(self):
        self.mybox.show_all()
    
                    
